@@ -13,8 +13,9 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, write to the 
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include "config.h"
 
@@ -25,10 +26,9 @@
 #include <stdlib.h>
 #include <time.h>
 #include <fcntl.h>
-#include <sys/mman.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 #include <gd.h>
 #endif
 
@@ -80,6 +80,8 @@ static const struct eeprom_info {
 	{ "MXIC MX25L8005A", 0xc21420c2, 1048576, 1 },
 	{ "MXIC MX25L1605A", 0xc21520c2, 2097152, 1 },
 
+	{ "Nantronics N25S80", 0xd51430d5, 1048576, 1 },                                                                                                                                     
+
 	{ "PMC Pm25LV010", 0x007e9d7f, 524288, 0 },
 
 	{ "Spansion S25FL004A", 0x00120201,  524288, 0 },
@@ -217,7 +219,8 @@ ax203_get_checksum(Camera *camera, int address, int size)
 			      (char *)buf, 2);
 	if (ret < 0) return ret;
 
-	return be16atoh(buf);
+	/*return be16atoh(buf);*/
+	return (buf[0] << 8) | buf[1];
 }
 
 static int
@@ -255,7 +258,8 @@ ax3003_get_abfs_start(Camera *camera)
 			      (char *)buf, 2);
 	if (ret < 0) return ret;
 
-	return be16atoh(buf) * 0x100;
+	return ((buf[0] << 8) | buf[1]) * 0x100;
+	/*return be16atoh(buf) * 0x100;*/
 }
 
 int
@@ -300,7 +304,7 @@ ax203_eeprom_device_identification(Camera *camera, char *buf)
 {
 	char cmd = SPI_EEPROM_RDID;
 
-	return ax203_send_eeprom_cmd (camera, 0, &cmd, 1, buf, 4, 0);
+	return ax203_send_eeprom_cmd (camera, 0, &cmd, 1, buf, 64, 0);
 }
 
 static int
@@ -645,6 +649,7 @@ static int ax203_read_parameter_block(Camera *camera)
 		if (i < 0) return i;
 		switch (i) {
 		case 0:
+		case 1:
 			camera->pl->width  = 320;
 			camera->pl->height = 240;
 			break;
@@ -852,7 +857,8 @@ int ax203_read_v3_3_x_v3_4_x_fileinfo(Camera *camera, int idx,
 					AX203_ABFS_FILE_OFFSET (idx),
 			       buf, 2))
 
-	fileinfo->address = le16atoh (buf) << 8;
+	/*fileinfo->address = le16atoh (buf) << 8;*/
+	fileinfo->address = (buf[0] | (buf[1] << 8)) << 8;
 	fileinfo->size    = ax203_filesize (camera);
 	fileinfo->present = fileinfo->address? 1 : 0;
 
@@ -873,7 +879,9 @@ int ax203_write_v3_3_x_v3_4_x_fileinfo(Camera *camera, int idx,
 	if (!fileinfo->present)
 		fileinfo->address = 0;
 
-	htole16a(buf, fileinfo->address >> 8);
+	/*htole16a(buf, fileinfo->address >> 8);*/
+	buf[0] = (fileinfo->address >> 8) & 0xff;
+	buf[1] = (fileinfo->address >>16) & 0xff;
 	CHECK (ax203_write_mem (camera,
 				camera->pl->fs_start +
 					AX203_ABFS_FILE_OFFSET (idx),
@@ -929,6 +937,11 @@ int ax3003_read_v3_5_x_fileinfo(Camera *camera, int idx,
 					AX3003_ABFS_FILE_OFFSET (idx),
 			       &raw, sizeof(raw)))
 
+	if (raw.address == 0xffff || raw.size == 0xffff) {
+		memset(fileinfo, 0, sizeof(*fileinfo));
+		return GP_OK;
+	}
+
 	fileinfo->present = raw.address && raw.size;
 	fileinfo->address = be16toh (raw.address) * 0x100;
 	fileinfo->size    = be16toh (raw.size) * 0x100;
@@ -1060,7 +1073,7 @@ ax203_file_present(Camera *camera, int idx)
 static int
 ax203_decode_image(Camera *camera, char *src, int src_size, int **dest)
 {
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 	int ret;
 	unsigned int x, y, width, height, row_skip = 0;
 	unsigned char *components[3];
@@ -1182,7 +1195,7 @@ ax203_decode_image(Camera *camera, char *src, int src_size, int **dest)
 static int
 ax203_encode_image(Camera *camera, int **src, char *dest, int dest_size)
 {
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 	int size = ax203_filesize (camera);
 #ifdef HAVE_LIBJPEG
 	int x, y;
@@ -1745,7 +1758,11 @@ ax203_open_device(Camera *camera)
 	/* Not sure if this is necessary, but the windows software does it */
 	CHECK (ax203_eeprom_release_from_deep_powerdown (camera))
 	CHECK (ax203_eeprom_device_identification (camera, buf))
-	id = le32atoh((uint8_t *)buf);
+	/*id = le32atoh((uint8_t *)buf);*/
+	id =	 (uint8_t)buf[0]	|
+		((uint8_t)buf[1] << 8)	|
+		((uint8_t)buf[2] << 16)	|
+		((uint8_t)buf[3] << 24);
 	for (i = 0; ax203_eeprom_info[i].name; i++) {
 		if (ax203_eeprom_info[i].id == id)
 			break;
