@@ -3,10 +3,10 @@
  * library.c
  *
  *   Canon Camera library for the gphoto project,
- *    1999 Wolfgang G. Reissnegger
+ *   Copyright 1999 Wolfgang G. Reissnegger
  *   Developed for the Canon PowerShot A50
  *   Additions for PowerShot A5 by Ole W. Saastad
- *    2000: Other additions  by Edouard Lafargue, Philippe Marzouk
+ *   Copyright 2000: Other additions  by Edouard Lafargue, Philippe Marzouk
  *
  * This file contains all the "glue code" required to use the canon
  * driver with libgphoto2.
@@ -20,16 +20,16 @@
  *
  ****************************************************************************/
 
+#define _BSD_SOURCE
+
 #include "config.h"
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <termios.h>
 #include <time.h>
 #include <ctype.h>
 
@@ -179,6 +179,7 @@ static const struct canonShutterSpeedStateStruct shutterSpeedStateArray[] = {
 };
 
 static const struct canonApertureStateStruct apertureStateArray[] = {
+	{APERTURE_F1_2, "1.2"},
 	{APERTURE_F1_4, "1.4"},
 	{APERTURE_F1_6, "1.6"},
 	{APERTURE_F1_8, "1.8"},
@@ -279,7 +280,7 @@ static const struct canonShootingModeStateStruct shootingModeStateArray[] = {
 	{0x73,N_("Flash off")},
 	{0x74,N_("Long shutter")},
 	{0x75,N_("Super macro")},
-	{0x76,N_("Foilage")},
+	{0x76,N_("Foliage")},
 	{0x77,N_("Indoor")},
 	{0x78,N_("Fireworks")},
 	{0x79,N_("Beach")},
@@ -297,6 +298,8 @@ static const struct canonShootingModeStateStruct shootingModeStateArray[] = {
 static const struct canonImageFormatStateStruct imageFormatStateArray[] = {
 	{IMAGE_FORMAT_RAW, N_("RAW"),
 	 0x04, 0x02, 0x00},
+	{IMAGE_FORMAT_RAW_2, N_("RAW 2"),
+	 0x03, 0x02, 0x00},
 	{IMAGE_FORMAT_SMALL_NORMAL_JPEG, N_("Small Normal JPEG"),
 	 0x02, 0x01, 0x02},
 	{IMAGE_FORMAT_SMALL_FINE_JPEG, N_("Small Fine JPEG"),
@@ -585,9 +588,6 @@ camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 	}
 	gp_file_set_data_and_size ( file, (char *)data, size );
 	gp_file_set_mime_type (file, GP_MIME_JPEG);	/* always */
-	/* Add an arbitrary file name so caller won't crash */
-	gp_file_set_name (file, "canon_preview.jpg");
-
 	return GP_OK;
 }
 
@@ -785,6 +785,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			ret = canon_int_get_file (camera, canon_path, &data, &datalen,
 						  context);
 			if (ret == GP_OK) {
+				/* 0 also marks image as downloaded */
 				uint8_t attr = 0;
 
 				/* This should cover all attribute
@@ -794,8 +795,6 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 				CameraFileInfo info;
 
 				gp_filesystem_get_info (fs, folder, filename, &info, context);
-				if (info.file.status == GP_FILE_STATUS_NOT_DOWNLOADED)
-					attr &= ~CANON_ATTR_DOWNLOADED;
 				if ((info.file.permissions & GP_FILE_PERM_DELETE) == 0)
 					attr |= CANON_ATTR_WRITE_PROTECTED;
 				canon_int_set_file_attributes (camera, filename,
@@ -803,6 +802,10 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 										 folder,
 										 context),
 							       attr, context);
+
+				/* we cannot easily retrieve it except by reading the directory */
+				if (info.file.fields & GP_FILE_INFO_MTIME)
+					gp_file_set_mtime (file, info.file.mtime);
 			}
 			break;
 
@@ -821,7 +824,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			thumbname = canon_int_filename2thumbname (camera, canon_path);
 			if (thumbname == NULL) {
 				/* no thumbnail available */
-				GP_DEBUG (_("%s is a file type for which no thumbnail is provided"),canon_path);
+				GP_DEBUG ("%s is a file type for which no thumbnail is provided",canon_path);
 				return (GP_ERROR_NOT_SUPPORTED);
 			}
 #ifdef HAVE_LIBEXIF
@@ -859,7 +862,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			thumbname = canon_int_filename2thumbname (camera, canon_path);
 			if (thumbname == NULL) {
 				/* no thumbnail available */
-				GP_DEBUG (_("%s is a file type for which no thumbnail is provided"), canon_path);
+				GP_DEBUG ("%s is a file type for which no thumbnail is provided", canon_path);
 				return (GP_ERROR_NOT_SUPPORTED);
 			}
 
@@ -903,7 +906,6 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	}
 
 	/* do different things with the data fetched above */
-	/* FIXME: For which file type(s) should we gp_file_set_name(file,filename) ? */
 	switch (type) {
 		case GP_FILE_TYPE_PREVIEW:
 			/* Either this camera model does not support EXIF,
@@ -932,19 +934,16 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 
 			gp_file_set_data_and_size (file, (char *)data, datalen);
 			gp_file_set_mime_type (file, GP_MIME_JPEG);	/* always */
-			gp_file_set_name (file, filename);
 			break;
 
 		case GP_FILE_TYPE_AUDIO:
 			gp_file_set_mime_type (file, GP_MIME_WAV);
 			gp_file_set_data_and_size (file, (char *)data, datalen);
-			gp_file_set_name (file, filename);
 			break;
 
 		case GP_FILE_TYPE_NORMAL:
 			gp_file_set_mime_type (file, filename2mimetype (filename));
 			gp_file_set_data_and_size (file, (char *)data, datalen);
-			gp_file_set_name (file, filename);
 			break;
 #ifdef HAVE_LIBEXIF
 		case GP_FILE_TYPE_EXIF:
@@ -956,9 +955,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			break;
 #endif /* HAVE_LIBEXIF */
 		default:
-			/* this case should've been caught above anyway */
-			if (data)
-				free (data);
+			free (data);
 			data = NULL;
 			return (GP_ERROR_NOT_SUPPORTED);
 	}
@@ -1315,7 +1312,8 @@ convert_filename_to_8_3(const char* filename, char* dest)
 
 /* XXX This function should be merged with the other one of the same name */
 static int
-put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void *data,
+put_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
+	       CameraFileType type, CameraFile *file, void *data,
 	       GPContext *context)
 {
 	Camera *camera = data;
@@ -1372,9 +1370,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void
 		sprintf (destpath, "%s\\%s", dcf_root_dir, dir);
 	} else {
 		if(camera->pl->upload_keep_filename) {
-		    const char* filename;
 		    char filename2[300];
-		    CHECK_RESULT (gp_file_get_name (file, &filename));
 		    if(!filename)
 			return GP_ERROR;
 		    convert_filename_to_8_3(filename, filename2);
@@ -1436,14 +1432,14 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void
 
 	clear_readiness (camera);
 
-	return canon_int_put_file (camera, file, destname, destpath, context);
+	return canon_int_put_file (camera, file, filename, destname, destpath, context);
 }
 
 #else /* not CANON_EXPERIMENTAL_UPLOAD */
 
 static int
-put_file_func (CameraFilesystem __unused__ *fs, const char __unused__ *folder,
-	       CameraFile *file, void *data,
+put_file_func (CameraFilesystem __unused__ *fs, const char __unused__ *folder, const char *filename,
+	       CameraFileType type, CameraFile *file, void *data,
 	       GPContext *context)
 {
 	Camera *camera = data;
@@ -1454,6 +1450,8 @@ put_file_func (CameraFilesystem __unused__ *fs, const char __unused__ *folder,
 	CameraAbilities a;
 
 	GP_DEBUG ("camera_folder_put_file()");
+	if (type != GP_FILE_TYPE_NORMAL)
+		return GP_ERROR_BAD_PARAMETERS;
 
 	if (camera->port->type == GP_PORT_USB) {
 		gp_context_error (context,
@@ -1548,7 +1546,7 @@ put_file_func (CameraFilesystem __unused__ *fs, const char __unused__ *folder,
 
 	clear_readiness (camera);
 
-	return canon_int_put_file (camera, file, destname, destpath, context);
+	return canon_int_put_file (camera, file, filename, destname, destpath, context);
 }
 
 #endif /* CANON_EXPERIMENTAL_UPLOAD */
@@ -1557,6 +1555,14 @@ put_file_func (CameraFilesystem __unused__ *fs, const char __unused__ *folder,
 
 /****************************************************************************/
 
+/* Wait for an event */
+static int
+camera_wait_for_event (Camera *camera, int timeout, CameraEventType *eventtype, void **eventdata, GPContext *context) 
+{
+	return canon_int_wait_for_event (camera, timeout, eventtype, eventdata, context);
+}
+
+
 /* Get configuration into screen widgets for display. */
 
 static int
@@ -1565,12 +1571,11 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	CameraWidget *t, *section;
 	char power_str[128], firm[64];
 
-	int iso, shutter_speed, aperture, focus_mode, flash_mode, beep_mode, shooting_mode;
-	int res_byte1, res_byte2, res_byte3;
 	int pwr_status, pwr_source, res, i, menuval;
-	unsigned int expbias;
 	time_t camtime;
 	char formatted_camera_time[30];
+	float zoom;
+	unsigned char zoomVal, zoomMax;
 
 	GP_DEBUG ("camera_get_config()");
 
@@ -1621,24 +1626,24 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 			return -1;
 	}
 
+	res = canon_int_get_release_params(camera, context);
+	if (res != GP_OK) {
+		for (i = 0 ; i < RELEASE_PARAMS_LEN ; i++)
+			camera->pl->release_params[i] = -1;
+	}
+
+
 	/* ISO speed */
 	gp_widget_new (GP_WIDGET_MENU, _("ISO Speed"), &t);
 	gp_widget_set_name (t, "iso");
 
-	/* Get the camera's current ISO Speed setting */
-	iso = -1;
-	if (camera->pl->cached_ready == 1) {
-		res = canon_int_get_release_params(camera, context);
-		if (res == GP_OK) 
-			iso = camera->pl->release_params[ISO_INDEX];
-	}
-
-	/* Map it to the list of choices */
+	/* Map ISO speed to the list of choices */
 	i = 0;
 	menuval = -1;
 	while (isoStateArray[i].label) {
 		gp_widget_add_choice (t, isoStateArray[i].label);
-		if (iso == (int)isoStateArray[i].value) {
+		if (camera->pl->release_params[ISO_INDEX] ==
+		    (int)isoStateArray[i].value) {
 			gp_widget_set_value (t, isoStateArray[i].label);
 			menuval = i;
 		}
@@ -1659,20 +1664,13 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	gp_widget_new (GP_WIDGET_MENU, _("Shooting mode"), &t);
 	gp_widget_set_name (t, "shootingmode");
 
-	/* Get the camera's current shooting mode setting */
-	shooting_mode = -1;
-	if (camera->pl->cached_ready == 1) {
-		res = canon_int_get_release_params(camera, context);
-		if (res == GP_OK)
-			iso = camera->pl->release_params[SHOOTING_MODE_INDEX];
-	}
-
-	/* Map it to the list of choices */
+	/* Map shooting mode to the list of choices */
 	i = 0;
 	menuval = -1;
 	while (shootingModeStateArray[i].label) {
 		gp_widget_add_choice (t, _(shootingModeStateArray[i].label));
-		if (shooting_mode == (int)shootingModeStateArray[i].value) {
+		if (camera->pl->release_params[SHOOTING_MODE_INDEX] ==
+		    (int)shootingModeStateArray[i].value) {
 			gp_widget_set_value (t, _(shootingModeStateArray[i].label));
 			menuval = i;
 		}
@@ -1693,20 +1691,13 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	gp_widget_new (GP_WIDGET_MENU, _("Shutter Speed"), &t);
 	gp_widget_set_name (t, "shutterspeed");
 
-	/* Get the camera's current shutter speed setting */
-	shutter_speed = -1;
-	if (camera->pl->cached_ready == 1) {
-		res = canon_int_get_release_params(camera, context);
-		if (res == GP_OK) 
-			shutter_speed = camera->pl->release_params[SHUTTERSPEED_INDEX];
-	}
-
-	/* Map it to the list of choices */
+	/* Map shutter speed to the list of choices */
 	i = 0;
 	menuval = -1;
 	while (shutterSpeedStateArray[i].label) {
 		gp_widget_add_choice (t, _(shutterSpeedStateArray[i].label));
-		if (shutter_speed == (int)shutterSpeedStateArray[i].value) {
+		if (camera->pl->release_params[SHUTTERSPEED_INDEX] ==
+		    (int)shutterSpeedStateArray[i].value) {
 			gp_widget_set_value (t, _(shutterSpeedStateArray[i].label));
 			menuval = i;
 		}
@@ -1724,36 +1715,29 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 
 
 
-	/* Zoom level */
-	gp_widget_new (GP_WIDGET_RANGE, _("Zoom"), &t);
-	gp_widget_set_name (t, "zoom");
-	gp_widget_set_range (t, 0, 255, 1);
-	/* Set an unknown zoom level (at the moment we don't read the
-	 * zoom level */
-	float zoom = -1;
-	gp_widget_set_value (t, &zoom);
-	gp_widget_append (section, t);
-
-
+	/* DSLRs have only "Manual" Zoom */
+        if ( camera->pl->md->id_str && !strstr(camera->pl->md->id_str,"EOS") && !strstr(camera->pl->md->id_str,"Rebel")) {
+		/* Zoom level */
+		gp_widget_new (GP_WIDGET_RANGE, _("Zoom"), &t);
+		gp_widget_set_name (t, "zoom");
+		canon_int_get_zoom(camera, &zoomVal, &zoomMax, context);
+		gp_widget_set_range (t, 0, zoomMax, 1);
+		zoom = zoomVal;
+		gp_widget_set_value (t, &zoom);
+		gp_widget_append (section, t);
+	}
 
 	/* Aperture */
 	gp_widget_new (GP_WIDGET_MENU, _("Aperture"), &t);
 	gp_widget_set_name (t, "aperture");
 
-	/* Get the camera's current aperture setting */
-	aperture = -1;
-	if (camera->pl->cached_ready == 1) {
-		res = canon_int_get_release_params(camera, context);
-		if (res == GP_OK) 
-			aperture = camera->pl->release_params[APERTURE_INDEX];
-	}
-
-	/* Map it to the list of choices */
+	/* Map aperture to the list of choices */
 	i = 0;
 	menuval = -1;
 	while (apertureStateArray[i].label) {
 		gp_widget_add_choice (t, _(apertureStateArray[i].label));
-		if (aperture == (int)apertureStateArray[i].value) {
+		if (camera->pl->release_params[APERTURE_INDEX] ==
+		    (int)apertureStateArray[i].value) {
 			gp_widget_set_value (t, _(apertureStateArray[i].label));
 			menuval = i;
 		}
@@ -1773,20 +1757,14 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	gp_widget_new (GP_WIDGET_MENU, _("Exposure Compensation"), &t);
 	gp_widget_set_name (t, "exposurecompensation");
 
-	/* Get the camera's current exposure compensation setting */
-	expbias = -1;
-	if (camera->pl->cached_ready == 1) {
-		res = canon_int_get_release_params(camera, context);
-		if (res == GP_OK) 
-			expbias = camera->pl->release_params[EXPOSUREBIAS_INDEX];
-	}
 
-	/* Map it to the list of choices */
+	/* Map exposure compensation to the list of choices */
 	i = 0;
 	menuval = -1;
 	while (exposureBiasStateArray[i].label) {
 		gp_widget_add_choice (t, _(exposureBiasStateArray[i].label));
-		if (expbias == exposureBiasStateArray[i].value) {
+		if (camera->pl->release_params[EXPOSUREBIAS_INDEX] ==
+		    exposureBiasStateArray[i].value) {
 			gp_widget_set_value (t, _(exposureBiasStateArray[i].label));
 			menuval = i;
 		}
@@ -1806,25 +1784,18 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	gp_widget_new (GP_WIDGET_MENU, _("Image Format"), &t);
 	gp_widget_set_name (t, "imageformat");
 
-	/* Get the camera's current image format setting */
-	res_byte1 = res_byte2 = res_byte3 = -1;
-	if (camera->pl->cached_ready == 1) {
-		res = canon_int_get_release_params (camera, context);
-		if (res == GP_OK) {
-			res_byte1 = camera->pl->release_params[IMAGE_FORMAT_1_INDEX];
-			res_byte2 = camera->pl->release_params[IMAGE_FORMAT_2_INDEX];
-			res_byte3 = camera->pl->release_params[IMAGE_FORMAT_3_INDEX];
-		}
-	}
 
-	/* Map it to the list of choices */
+	/* Map image format to the list of choices */
 	i = 0;
 	menuval = -1;
 	while (imageFormatStateArray[i].label) {
 		gp_widget_add_choice (t, _(imageFormatStateArray[i].label));
-		if (res_byte1 == imageFormatStateArray[i].res_byte1 &&
-			res_byte2 == imageFormatStateArray[i].res_byte2 &&
-			res_byte3 == imageFormatStateArray[i].res_byte3) {
+		if ((camera->pl->release_params[IMAGE_FORMAT_1_INDEX] ==
+		     imageFormatStateArray[i].res_byte1) &&
+		    (camera->pl->release_params[IMAGE_FORMAT_2_INDEX] ==
+		     imageFormatStateArray[i].res_byte2) &&
+		    (camera->pl->release_params[IMAGE_FORMAT_3_INDEX] ==
+		     imageFormatStateArray[i].res_byte3)) {
 			gp_widget_set_value (t, _(imageFormatStateArray[i].label));
 			menuval = i;
 		}
@@ -1845,20 +1816,14 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	gp_widget_new (GP_WIDGET_MENU, _("Focus Mode"), &t);
 	gp_widget_set_name (t, "focusmode");
 
-	/* Get the camera's current focus mode setting */
-	focus_mode = -1;
-	if (camera->pl->cached_ready == 1) {
-		res = canon_int_get_release_params(camera, context);
-		if (res == GP_OK) 
-			focus_mode = camera->pl->release_params[FOCUS_MODE_INDEX];
-	}
 
-	/* Map it to the list of choices */
+	/* Map focus to the list of choices */
 	i = 0;
 	menuval = -1;
 	while (focusModeStateArray[i].label) {
 		gp_widget_add_choice (t, _(focusModeStateArray[i].label));
-		if (focus_mode == (int)focusModeStateArray[i].value) {
+		if (camera->pl->release_params[FOCUS_MODE_INDEX] ==
+		    (int)focusModeStateArray[i].value) {
 			gp_widget_set_value (t, _(focusModeStateArray[i].label));
 			menuval = i;
 		}
@@ -1879,27 +1844,21 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	gp_widget_new (GP_WIDGET_MENU, _("Flash Mode"), &t);
 	gp_widget_set_name (t, "flashmode");
 
-	/* Get the camera's current flash mode setting */
-	flash_mode = -1;
-	if (camera->pl->cached_ready == 1) {
-		res = canon_int_get_release_params(camera, context);
-		if (res == GP_OK) 
-			flash_mode = camera->pl->release_params[FLASH_INDEX];
-	}
 
-	/* Map it to the list of choices */
+	/* Map flash mode to the list of choices */
 	i = 0;
 	menuval = -1;
 	while (flashModeStateArray[i].label) {
 		gp_widget_add_choice (t, _(flashModeStateArray[i].label));
-		if (flash_mode == (int)flashModeStateArray[i].value) {
+		if (camera->pl->release_params[FLASH_INDEX] ==
+		    (int)flashModeStateArray[i].value) {
 			gp_widget_set_value (t, _(flashModeStateArray[i].label));
 			menuval = i;
 		}
 		i++;
 	}
 	
-	/* Set an unknown shutter value if the 
+	/* Set an unknown flash value if the 
 	 * camera is set to something weird */
 	if (menuval == -1) {
 		gp_widget_add_choice (t, _("Unknown"));
@@ -1913,20 +1872,14 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	gp_widget_new (GP_WIDGET_MENU, _("Beep"), &t);
 	gp_widget_set_name (t, "beep");
 
-	/* Get the camera's current beep setting */
-	beep_mode = -1;
-	if (camera->pl->cached_ready == 1) {
-		res = canon_int_get_release_params(camera, context);
-		if (res == GP_OK) 
-			beep_mode = camera->pl->release_params[BEEP_INDEX];
-	}
 
-	/* Map it to the list of choices */
+	/* Map beep mode to the list of choices */
 	i = 0;
 	menuval = -1;
 	while (beepModeStateArray[i].label) {
 		gp_widget_add_choice (t, _(beepModeStateArray[i].label));
-		if (beep_mode == (int)beepModeStateArray[i].value) {
+		if (camera->pl->release_params[BEEP_INDEX] ==
+		    (int)beepModeStateArray[i].value) {
 			gp_widget_set_value (t, _(beepModeStateArray[i].label));
 			menuval = i;
 		}
@@ -1969,17 +1922,13 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 
 	gp_widget_new (GP_WIDGET_TEXT, _("Date and Time"), &t);
 	gp_widget_set_name (t, "datetime");
-	if (camera->pl->cached_ready == 1) {
-		res = canon_int_get_time (camera, &camtime, context);
-		if (res == GP_OK) {
-			strftime (formatted_camera_time, sizeof (formatted_camera_time),
+	res = canon_int_get_time (camera, &camtime, context);
+	if (res == GP_OK) {
+		strftime (formatted_camera_time, sizeof (formatted_camera_time),
                           "%Y-%m-%d %H:%M:%S", gmtime (&camtime));
-			gp_widget_set_value (t, formatted_camera_time);
-		} else {
-			gp_widget_set_value (t, _("Error"));
-		}
+		gp_widget_set_value (t, formatted_camera_time);
 	} else {
-		gp_widget_set_value (t, _("Unavailable"));
+		gp_widget_set_value (t, _("Error"));
 	}
 	gp_widget_append (section, t);
 
@@ -1990,23 +1939,18 @@ camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 	gp_widget_set_value (t, firm);
 	gp_widget_append (section, t);
 
-	if (camera->pl->cached_ready == 1) {
-		canon_get_batt_status (camera, &pwr_status, &pwr_source, context);
+	canon_get_batt_status (camera, &pwr_status, &pwr_source, context);
 
-		if (pwr_status == CAMERA_POWER_OK || pwr_status == CAMERA_POWER_BAD)
-			snprintf (power_str, sizeof (power_str), "%s (%s)",
-				  ((pwr_source & CAMERA_MASK_BATTERY) ==
-				   0) ? _("AC adapter") : _("on battery"),
-				  pwr_status ==
-				  CAMERA_POWER_OK ? _("power OK") : _("power bad"));
-		else
-			snprintf (power_str, sizeof (power_str), "%s - %i",
-				  ((pwr_source & CAMERA_MASK_BATTERY) ==
-				   0) ? _("AC adapter") : _("on battery"), pwr_status);
-	} else {
-		strncpy (power_str, _("Unavailable"), sizeof (power_str) - 1);
-		power_str[sizeof (power_str) - 1] = 0;
-	}
+	if (pwr_status == CAMERA_POWER_OK || pwr_status == CAMERA_POWER_BAD)
+		snprintf (power_str, sizeof (power_str), "%s (%s)",
+			  ((pwr_source & CAMERA_MASK_BATTERY) ==
+			   0) ? _("AC adapter") : _("on battery"),
+			  pwr_status ==
+			  CAMERA_POWER_OK ? _("power OK") : _("power bad"));
+	else
+		snprintf (power_str, sizeof (power_str), "%s - %i",
+			  ((pwr_source & CAMERA_MASK_BATTERY) ==
+			   0) ? _("AC adapter") : _("on battery"), pwr_status);
 
 	gp_widget_new (GP_WIDGET_TEXT, _("Power"), &t);
 	gp_widget_set_name (t, "power");
@@ -2315,21 +2259,23 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 	}
 
 
-	/* Zoom */
-	gp_widget_get_child_by_label (window, _("Zoom"), &w);
-	if (gp_widget_changed (w)) {
-		float zoom;
-		gp_widget_get_value (w, &zoom);
-		if (!check_readiness (camera, context)) {
-			gp_context_status (context, _("Camera unavailable"));
-		} else {
-				if (canon_int_set_zoom (camera, zoom, context) == GP_OK)
-					gp_context_status (context, _("Zoom level changed"));
-				else
-					gp_context_status (context, _("Could not change zoom level"));
-			}		
-		}
-
+	/* DSLRs have only "Manual" Zoom */
+        if ( camera->pl->md->id_str && !strstr(camera->pl->md->id_str,"EOS") && !strstr(camera->pl->md->id_str,"Rebel")) {
+		/* Zoom */
+		gp_widget_get_child_by_label (window, _("Zoom"), &w);
+		if (gp_widget_changed (w)) {
+			float zoom;
+			gp_widget_get_value (w, &zoom);
+			if (!check_readiness (camera, context)) {
+				gp_context_status (context, _("Camera unavailable"));
+			} else {
+					if (canon_int_set_zoom (camera, zoom, context) == GP_OK)
+						gp_context_status (context, _("Zoom level changed"));
+					else
+						gp_context_status (context, _("Could not change zoom level"));
+				}		
+			}
+	}
 
 	/* Aperture */
 	gp_widget_get_child_by_label (window, _("Aperture"), &w);
@@ -2436,6 +2382,7 @@ get_info_func (CameraFilesystem __unused__ *fs, const char *folder,
 	       CameraFileInfo * info, 
 	       void __unused__ *data, GPContext __unused__ *context)
 {
+	Camera *camera = data;
 	GP_DEBUG ("get_info_func() called for '%s'/'%s'", folder, filename);
 
 	info->preview.fields = GP_FILE_INFO_TYPE;
@@ -2444,7 +2391,7 @@ get_info_func (CameraFilesystem __unused__ *fs, const char *folder,
 	strcpy (info->preview.type, GP_MIME_JPEG);
 
 	/* FIXME GP_FILE_INFO_PERMISSIONS to add */
-	info->file.fields = GP_FILE_INFO_NAME | GP_FILE_INFO_TYPE;
+	info->file.fields = GP_FILE_INFO_TYPE;
 	/* | GP_FILE_INFO_PERMISSIONS | GP_FILE_INFO_SIZE; */
 	/* info->file.fields.permissions =  */
 
@@ -2455,12 +2402,9 @@ get_info_func (CameraFilesystem __unused__ *fs, const char *folder,
 	else if (is_audio (filename))
 		strcpy (info->file.type, GP_MIME_WAV);
 	else
-		/* May not be correct behaviour ... */
-		strcpy (info->file.type, "unknown");
-
-	strcpy (info->file.name, filename);
-
-	return GP_OK;
+		strcpy (info->file.type, GP_MIME_UNKNOWN);
+	/* let it fill driver specific info */
+	return canon_int_get_info_func (camera, folder, filename, info, context);
 }
 
 static int
@@ -2590,6 +2534,7 @@ camera_init (Camera *camera, GPContext *context)
 	camera->functions->summary = camera_summary;
 	camera->functions->manual = camera_manual;
 	camera->functions->about = camera_about;
+	camera->functions->wait_for_event = camera_wait_for_event;
 
 	/* Set up the CameraFilesystem */
 	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
