@@ -70,23 +70,23 @@ digi_first_decompress (unsigned char *output, unsigned char *input,
         	while (parity < 2 ) {
 			while ( lookup > table[cycles]) {
 				if (bit_counter == 8) {
-        				input_byte = input[bytes_used];
-        				bytes_used ++;
+					input_byte = input[bytes_used];
+					bytes_used ++;
 					temp1 = input_byte;
-        				bit_counter = 0;
-        			}
+					bit_counter = 0;
+				}
 				input_byte = temp1;
 				temp2 = (temp2 << 1) & 0xFF;
 				input_byte = input_byte >> 7;
 				temp2 = temp2 | input_byte;
-				temp1 = (temp1 <<1)&0xFF;
+				temp1 = (temp1 <<1) & 0xFF;
 				bit_counter ++ ;
 				cycles ++ ;
 				if (cycles > 8) {
 					GP_DEBUG ("Too many cycles?\n");
 					return GP_ERROR; 
 				}
-				lookup = temp2 & 0xff;        
+				lookup = temp2 & 0xff;
 			}
 			temp2 = 0;
 			for (i=0; i < 17; i++ ) {
@@ -99,26 +99,26 @@ digi_first_decompress (unsigned char *output, unsigned char *input,
 					nibble_to_keep[parity] = translator[i];
 					break;	
 				}
-			}		
+			}
 			cycles = 0;
 			parity ++ ;
 		} 
 		output[bytes_done] = (nibble_to_keep[0] << 4)
 						| nibble_to_keep[1];
-		bytes_done ++ ;
-		parity = 0; 
+		bytes_done++;
+		parity = 0;
 	}
-	GP_DEBUG ("bytes_used = 0x%x = %i\n", bytes_used, bytes_used);        
+	GP_DEBUG ("bytes_used = 0x%x = %i\n", bytes_used, bytes_used);
 	return GP_OK;
 }
 
 static int
-digi_second_decompress (unsigned char *uncomp, unsigned char *in, 
+digi_second_decompress (unsigned char *uncomp, unsigned char *in,
 						    int width, int height)
 {
 	int diff = 0;
 	int tempval = 0;
-	int i, m, parity;
+	int i, m;
 	unsigned char delta_left = 0;
 	unsigned char delta_right = 0;
 	int input_counter = 0;
@@ -130,89 +130,91 @@ digi_second_decompress (unsigned char *uncomp, unsigned char *in,
 	templine_red = malloc(width);
 	if (!templine_red) {
 		return GP_ERROR_NO_MEMORY;
-	}	
-	for(i=0; i < width; i++){
+	}
+	for(i=0; i < width; i++)
 	    templine_red[i] = 0x80;
-	}
-	templine_green = malloc(width);	
+
+	templine_green = malloc(width);
 	if (!templine_green) {
+		free (templine_red);
 		return GP_ERROR_NO_MEMORY;
-	}	
-	for(i=0; i < width; i++){
+	}
+	for(i=0; i < width; i++)
 		templine_green[i] = 0x80;
-	}
-	templine_blue = malloc(width);	
+	
+	templine_blue = malloc(width);
 	if (!templine_blue) {
+		free (templine_red);
+		free (templine_green);
 		return GP_ERROR_NO_MEMORY;
-	}	
-	for(i=0; i < width; i++){
-		templine_blue[i] = 0x80;
 	}
+	for(i=0; i < width; i++)
+		templine_blue[i] = 0x80;
+
 	GP_DEBUG ("Running second_decompress.\n");
-	for (m=0; m < height/2; m++) {
+	for (m = 0; m < height / 2; m++) {
 		/* First we do an even-numbered line */
-		for (i=0; i< width/2; i++) {
-			parity = i&1;
-			delta_right = in[input_counter] &0x0f;
-			delta_left = (in[input_counter]>>4)&0xff;
-			input_counter ++;
+		for (i = 0; i < width / 2; i++) {
+			delta_right = in[input_counter] & 0x0f;
+			delta_left = (in[input_counter] >> 4) & 0xff;
+			input_counter++;
 			/* left pixel (red) */
 			diff = delta_table[delta_left];
-			if (!i) 
+			if (!i)
 				tempval = templine_red[0] + diff;
-			else 
+			else
 				tempval = (templine_red[i]
 				    + uncomp[2 *m * width + 2 * i - 2]) / 2
 								    + diff;
 			tempval = MIN(tempval, 0xff);
 			tempval = MAX(tempval, 0);
-			uncomp[2*m*width+2*i] = tempval;
+			uncomp[2 * m * width + 2 * i] = tempval;
 			templine_red[i] = tempval;
 			/* right pixel (green) */
 			diff = delta_table[delta_right];
-			if (!i) 
+			if (!i)
 				tempval = templine_green[1] + diff;
-			else if (2*i == width - 2 ) 
+			else if (2 * i == width - 2 )
 				tempval = (templine_green[i]
-						+ uncomp[2*m*width+2*i-1])/2 
-							+ diff;
+				    + uncomp[2 * m * width + 2 * i -1]) / 2
+								    + diff;
 			else
-				tempval = (templine_green[i+1]
-						+ uncomp[2*m*width+2*i-1])/2 
-							+ diff;
+				tempval = (templine_green[i + 1]
+				    + uncomp[2 * m * width + 2 * i - 1]) / 2
+								    + diff;
 			tempval = MIN(tempval, 0xff);
 			tempval = MAX(tempval, 0);
-			uncomp[2*m*width+2*i+1] = tempval;
+			uncomp[2 * m * width + 2 * i + 1] = tempval;
 			templine_green[i] = tempval;
 		}
 		/* then an odd-numbered line */
-		for (i=0; i< width/2; i++) {
+		for (i = 0; i < width / 2; i++) {
 			delta_right = in[input_counter] &0x0f;
-			delta_left = (in[input_counter]>>4)&0xff;
-			input_counter ++;
+			delta_left = (in[input_counter] >> 4) & 0xff;
+			input_counter++;
 			/* left pixel (green) */
 			diff = delta_table[delta_left];
-			if (!i) 
+			if (!i)
 				tempval = templine_green[0] + diff;
-			else 
+			else
 				tempval = (templine_green[i]
 				    + uncomp[(2 * m + 1) * width
 						+ 2 * i - 2]) / 2 + diff;
 			tempval = MIN(tempval, 0xff);
 			tempval = MAX(tempval, 0);
-			uncomp[(2*m+1)*width+2*i] = tempval;
+			uncomp[(2 * m + 1) * width + 2 * i] = tempval;
 			templine_green[i] = tempval;
 			/* right pixel (blue) */
 			diff = delta_table[delta_right];
-			if (!i) 
+			if (!i)
 				tempval = templine_blue[0] + diff;
-			else 
+			else
 				tempval = (templine_blue[i]
 				    + uncomp[(2 * m + 1) * width
 						+ 2 * i - 1]) / 2 + diff;
 			tempval = MIN(tempval, 0xff);
 			tempval = MAX(tempval, 0);
-			uncomp[(2*m+1)*width+2*i+1] = tempval;
+			uncomp[(2 * m + 1) * width + 2 * i + 1] = tempval;
 			templine_blue[i] = tempval;
 		}
 	}
@@ -224,13 +226,13 @@ digi_second_decompress (unsigned char *uncomp, unsigned char *in,
 
 int
 digi_decompress (unsigned char *out_data, unsigned char *data,
-	    int w, int h)
+							int w, int h)
 {
 	int size;
 	unsigned char *temp_data;
-	size = w*h/2;
+	size = w * h / 2;
 	temp_data = malloc(size);
-	if (!temp_data) 
+	if (!temp_data)
 		return GP_ERROR_NO_MEMORY;
 	digi_first_decompress (temp_data, data, size);
 	GP_DEBUG("Stage one done\n");
@@ -240,8 +242,8 @@ digi_decompress (unsigned char *out_data, unsigned char *data,
 	return(GP_OK);
 }
 
-/* Brightness correction routine adapted from 
- * camlibs/polaroid/jd350e.c, copyright  2001 Michael Trawny 
+/* Brightness correction routine adapted from
+ * camlibs/polaroid/jd350e.c, copyright  2001 Michael Trawny
  * <trawny99@users.sourceforge.net>
  */
 
@@ -263,17 +265,17 @@ digi_decompress (unsigned char *out_data, unsigned char *data,
 #define CLAMP(x)        ((x)<0?0:((x)>255)?255:(x))
 #endif
 
-int 
-digi_postprocess(int width, int height, 
+int
+digi_postprocess(int width, int height,
 					unsigned char* rgb)
 {
 	int
 		x,y,
-		red_min=255, red_max=0, 
-		blue_min=255, blue_max=0, 
+		red_min=255, red_max=0,
+		blue_min=255, blue_max=0,
 		green_min=255, green_max=0;
 	double
-		min, max, amplify; 
+		min, max, amplify;
 
 	/* determine min and max per color... */
 
@@ -289,9 +291,9 @@ digi_postprocess(int width, int height,
 
 	for( y=0; y<height; y++){
 		for( x=0; x<width; x++ ){
-			MINMAX( RED(rgb,x,y,width), red_min,   red_max  );
-			MINMAX( GREEN(rgb,x,y,width), green_min, green_max);
-			MINMAX( BLUE(rgb,x,y,width), blue_min,  blue_max );
+			MINMAX( RED(rgb , x , y, width), red_min, red_max);
+			MINMAX( GREEN(rgb, x, y, width), green_min, green_max);
+			MINMAX( BLUE(rgb, x, y, width), blue_min, blue_max);
 		}
 	}
 
@@ -301,8 +303,8 @@ digi_postprocess(int width, int height,
 	min = MIN( MIN( red_min, green_min ), blue_min);
 	amplify = 255.0/(max-min);
 
-	for( y=0; y<height; y++){
-		for( x=0; x<width; x++ ){
+	for (y = 0; y < height; y++){
+		for(x = 0; x < width; x++ ){
 			RED(rgb, x, y, width)= MIN(amplify *
 				  (double)(RED(rgb, x, y, width) - min), 255);
 			GREEN(rgb, x, y, width)= MIN(amplify *
@@ -336,17 +338,17 @@ histogram (unsigned char *data, unsigned int size, int *htable_r,
 {
 	int x;
 	/* Initializations */
-	for (x = 0; x < 0x100; x++) { 
-		htable_r[x] = 0; 
-		htable_g[x] = 0; 
-		htable_b[x] = 0; 
+	for (x = 0; x < 0x100; x++) {
+		htable_r[x] = 0;
+		htable_g[x] = 0;
+		htable_b[x] = 0;
 	}
 	/* Building the histograms */
 	for (x = 0; x < (size * 3); x += 3)
 	{
-		htable_r[data[x+0]]++;	/* red histogram */
-		htable_g[data[x+1]]++;	/* green histogram */
-		htable_b[data[x+2]]++;	/* blue histogram */
+		htable_r[data[x + 0]]++;	/* red   histogram */
+		htable_g[data[x + 1]]++;	/* green histogram */
+		htable_b[data[x + 2]]++;	/* blue  histogram */
 	}
 	return GP_OK;
 }
@@ -366,14 +368,14 @@ white_balance (unsigned char *data, unsigned int size, float saturation)
 	x = 1;
 	for (r = 64; r < 192; r++)
 	{
-		x += htable_r[r]; 
+		x += htable_r[r];
 		x += htable_g[r];
 		x += htable_b[r];
 	}
 	new_gamma = sqrt((double) (x * 1.5) / (double) (size * 3));
 	GP_DEBUG("Provisional gamma correction = %1.2f\n", new_gamma);
 	/* Recalculate saturation factor for later use. */
-	saturation=saturation*new_gamma*new_gamma;
+	saturation = saturation * new_gamma * new_gamma;
 	GP_DEBUG("saturation = %1.2f\n", saturation);
 	gamma = new_gamma;
 	if (new_gamma < .70) gamma = 0.70;
@@ -381,18 +383,18 @@ white_balance (unsigned char *data, unsigned int size, float saturation)
 	GP_DEBUG("Gamma correction = %1.2f\n", gamma);
 	gp_gamma_fill_table(gtable, gamma);
 	gp_gamma_correct_single(gtable,data,size);
-	if (saturation < .5 ) /* If so, exit now. */ 
+	if (saturation < .5 ) /* If so, exit now. */
 		return GP_OK;
 
 	/* ---------------- BRIGHT DOTS ------------------- */
-	max = size / 200; 
+	max = size / 200;
 	histogram(data, size, htable_r, htable_g, htable_b);
 
-	for (r=0xfe, x=0; (r > 32) && (x < max); r--)  
-		x += htable_r[r]; 
-	for (g=0xfe, x=0; (g > 32) && (x < max); g--) 
+	for (r = 0xfe, x = 0; (r > 32) && (x < max); r--)
+		x += htable_r[r];
+	for (g = 0xfe, x = 0; (g > 32) && (x < max); g--)
 		x += htable_g[g];
-	for (b=0xfe, x=0; (b > 32) && (x < max); b--) 
+	for (b = 0xfe, x = 0; (b > 32) && (x < max); b--)
 		x += htable_b[b];
 	r_factor = (double) 0xfd / r;
 	g_factor = (double) 0xfd / g;
@@ -403,20 +405,20 @@ white_balance (unsigned char *data, unsigned int size, float saturation)
 	if (b_factor > max_factor) max_factor = b_factor;
 	if (max_factor >= 4.0) {
 	/* We need a little bit of control, here. If max_factor > 4 the photo
-	 * was very dark, after all. 
+	 * was very dark, after all.
 	 */
-		if (2.0*b_factor < max_factor)
-			b_factor = max_factor/2.;
-		if (2.0*r_factor < max_factor)
-			r_factor = max_factor/2.;
-		if (2.0*g_factor < max_factor)
-			g_factor = max_factor/2.;
+		if (2.0 * b_factor < max_factor)
+			b_factor = max_factor / 2.;
+		if (2.0 * r_factor < max_factor)
+			r_factor = max_factor / 2.;
+		if (2.0 * g_factor < max_factor)
+			g_factor = max_factor / 2.;
 		r_factor = (r_factor / max_factor) * 4.0;
 		g_factor = (g_factor / max_factor) * 4.0;
 		b_factor = (b_factor / max_factor) * 4.0;
 	}
 
-	if (max_factor > 1.5) 
+	if (max_factor > 1.5)
 		saturation = 0;
 	GP_DEBUG("White balance (bright): r=%1d, g=%1d, b=%1d, \
 			r_factor=%1.3f, g_factor=%1.3f, b_factor=%1.3f\n",
@@ -424,20 +426,20 @@ white_balance (unsigned char *data, unsigned int size, float saturation)
 	if (max_factor <= 1.4) {
 		for (x = 0; x < (size * 3); x += 3)
 		{
-			d = (data[x+0]<<8) * r_factor+8;
-			d >>=8;
+			d = (data[x + 0] << 8) * r_factor + 8;
+			d >>= 8;
 			if (d > 0xff) 
 				d = 0xff;
-			data[x+0] = d;
-			d = (data[x+1]<<8) * g_factor+8;
-			d >>=8;
+			data[x + 0] = d;
+			d = (data[x + 1] << 8) * g_factor + 8;
+			d >>= 8;
 			if (d > 0xff) { d = 0xff; }
-			data[x+1] = d;
-			d = (data[x+2]<<8) * b_factor+8;
-			d >>=8;
+			data[x + 1] = d;
+			d = (data[x + 2] << 8) * b_factor + 8;
+			d >>= 8;
 			if (d > 0xff) 
 				d = 0xff;
-			data[x+2] = d;
+			data[x + 2] = d;
 		}
 	}
 	/* ---------------- DARK DOTS ------------------- */
@@ -445,15 +447,15 @@ white_balance (unsigned char *data, unsigned int size, float saturation)
 	histogram(data, size, htable_r, htable_g, htable_b);
 
 	for (r = 0, x = 0; r < 96 && x < max; r++)
-		x += htable_r[r]; 
+		x += htable_r[r];
 	for (g = 0, x = 0; g < 96 && x < max; g++)
 		x += htable_g[g];
 	for (b = 0, x = 0; b < 96 && x < max; b++)
 		x += htable_b[b];
 
-	r_factor = (double) 0xfe / (0xff-r);
-	g_factor = (double) 0xfe / (0xff-g);
-	b_factor = (double) 0xfe / (0xff-b);
+	r_factor = (double) 0xfe / (0xff - r);
+	g_factor = (double) 0xfe / (0xff - g);
+	b_factor = (double) 0xfe / (0xff - b);
 
 	GP_DEBUG(
 	"White balance (dark): r=%1d, g=%1d, b=%1d, \
@@ -462,17 +464,17 @@ white_balance (unsigned char *data, unsigned int size, float saturation)
 
 	for (x = 0; x < (size * 3); x += 3)
 	{
-		d = (int) 0xff08-(((0xff-data[x+0])<<8) * r_factor);
+		d = (int) 0xff08 - (((0xff - data[x + 0]) << 8) * r_factor);
 		d >>= 8;
 		if (d < 0)
 			 d = 0;
-		data[x+0] = d;
-		d = (int) 0xff08-(((0xff-data[x+1])<<8) * g_factor);
+		data[x + 0] = d;
+		d = (int) 0xff08 - (((0xff - data[x + 1]) << 8) * g_factor);
 		d >>= 8;
 		if (d < 0)
 			d = 0;
-		data[x+1] = d;
-		d = (int) 0xff08-(((0xff-data[x+2])<<8) * b_factor);
+		data[x + 1] = d;
+		d = (int) 0xff08 - (((0xff - data[x + 2]) << 8) * b_factor);
 		d >>= 8;
 		if (d < 0)
 			d = 0;
@@ -489,13 +491,13 @@ white_balance (unsigned char *data, unsigned int size, float saturation)
 			if ( r > d )
 				r = r + (int) ((r - d) * (0xff - r)
 						/(0x100 - d) * saturation);
-			else 
+			else
 				r = r + (int) ((r - d) * (0xff - d)
 						/ (0x100 - r) * saturation);
 			if (g > d)
 				g = g + (int) ((g - d) * (0xff - g)
 						/ (0x100 - d) * saturation);
-			else 
+			else
 				g = g + (int) ((g - d) * (0xff - d)
 						/ (0x100 - g) * saturation);
 			if (b > d)
