@@ -1,7 +1,7 @@
 /* qm150.c
  *
- * Copyright  2003 Marcus Meissner <marcus@jet.franken.de>
- *                  Aurlien Croc (APC) <programming@ap2c.com>
+ * Copyright 2003 Marcus Meissner <marcus@jet.franken.de>
+ *                Aurelien Croc (AP2C) <programming@ap2c.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Modified by Aurlien Croc (APC) <programming@ap2c.com>
  * In particular : fix some bugs, and implementation of advanced 
@@ -166,7 +166,7 @@ k_ping (GPPort *port) {
  */
 static int
 k_info_img (unsigned int image_no, void *data, CameraFileInfo* info, 
-		unsigned int *data_number)
+		int *data_number)
 {
 	unsigned char	cmd[6], buf[INFO_BUFFER];
 	Camera *camera = data;
@@ -335,8 +335,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	       GPContext *context)
 {
 	unsigned char *d,*thumbnail;
-	unsigned int image_number;
-	int image_no, len, ret;
+	int image_number, image_no, len, ret;
 	long long_len;
 	CameraFileInfo file_info;
 	exifparser exifdat;
@@ -416,7 +415,7 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 	Camera *camera = data;
 	CameraFileInfo file_info;
 	unsigned char cmd[7], ack;
-	unsigned int image_no;
+	int image_no;
 	int ret;
 
 	GP_DEBUG ("*** ENTER: delete_file_func ***");
@@ -543,7 +542,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, const char *name,
 				return ret;
 			}
 			/* and complete with zeros */
-			bzero(buf,DATA_BUFFER);
+			memset(buf,0,DATA_BUFFER);
 			ret = gp_port_write (camera->port, (char*)buf, (DATA_BUFFER - 
 				(len - len_sent)));
 			if (ret<GP_OK) {
