@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 /*
  * This is working, but unfinished!
@@ -41,8 +41,12 @@
 #include <sys/select.h>
 #endif
 
-#include <sys/socket.h>
-#include <netinet/in.h>
+#ifdef WIN32
+# include <winsock.h>
+#else
+# include <sys/socket.h>
+# include <netinet/in.h>
+#endif
 
 #include <gphoto2/gphoto2-library.h>
 #include <gphoto2/gphoto2-port-log.h>
@@ -90,21 +94,30 @@
 #define ptpip_cmd_param4	30
 #define ptpip_cmd_param5	34
 
+#define PTP_EVENT_CHECK			0x0000	/* waits for */
+#define PTP_EVENT_CHECK_FAST		0x0001	/* checks */
 static uint16_t ptp_ptpip_check_event (PTPParams* params);
+static uint16_t ptp_ptpip_event (PTPParams* params, PTPContainer* event, int wait);
 
 /* send / receive functions */
 uint16_t
-ptp_ptpip_sendreq (PTPParams* params, PTPContainer* req)
+ptp_ptpip_sendreq (PTPParams* params, PTPContainer* req, int dataphase)
 {
-	unsigned int 		ret;
-	uint32_t		len = 18+req->Nparam*4;
-	unsigned char 		*request = malloc(len);
+	int		ret;
+	int		len = 18+req->Nparam*4;
+	unsigned char 	*request = malloc(len);
+
+	GP_LOG_D ("Sending PTP_OC 0x%0x (%s) request...", req->Code, ptp_get_opcode_name(params, req->Code));
 
 	ptp_ptpip_check_event (params);
 
 	htod32a(&request[ptpip_type],PTPIP_CMD_REQUEST);
 	htod32a(&request[ptpip_len],len);
-	htod32a(&request[ptpip_cmd_dataphase],1);	/* FIXME: dataphase handling */
+	/* sending data = 2, receiving data or no data = 1 */
+	if ((dataphase&PTP_DP_DATA_MASK) == PTP_DP_SENDDATA)
+		htod32a(&request[ptpip_cmd_dataphase],2);
+	else
+		htod32a(&request[ptpip_cmd_dataphase],1);
 	htod16a(&request[ptpip_cmd_code],req->Code);
 	htod32a(&request[ptpip_cmd_transid],req->Transaction_ID);
 
@@ -118,13 +131,13 @@ ptp_ptpip_sendreq (PTPParams* params, PTPContainer* req)
 	default:
 		break;
 	}
-	gp_log_data ( "ptpip/oprequest", (char*)request, len);
+	GP_LOG_DATA ( (char*)request, len, "ptpip/oprequest data:");
 	ret = write (params->cmdfd, request, len);
 	free (request);
 	if (ret == -1)
 		perror ("sendreq/write to cmdfd");
 	if (ret != len) {
-		gp_log (GP_LOG_ERROR, "ptpip","ptp_ptpip_sendreq() len =%d but ret=%d", len, ret);
+		GP_LOG_E ("ptp_ptpip_sendreq() len =%d but ret=%d", len, ret);
 		return PTP_RC_OK;
 	}
 	return PTP_RC_OK;
@@ -142,39 +155,39 @@ ptp_ptpip_generic_read (PTPParams *params, int fd, PTPIPHeader *hdr, unsigned ch
 			perror ("read PTPIPHeader");
 			return PTP_RC_GeneralError;
 		}
-		gp_log_data ( "ptpip/generic_read", (char*)xhdr+curread, ret);
+		GP_LOG_DATA ((char*)xhdr+curread, ret, "ptpip/generic_read header:");
 		curread += ret;
 		if (ret == 0) {
-			gp_log (GP_LOG_ERROR, "ptpip", "End of stream after reading %d bytes of ptpipheader", ret);
+			GP_LOG_E ("End of stream after reading %d bytes of ptpipheader", ret);
 			return PTP_RC_GeneralError;
 		}
 	}
 	len = dtoh32 (hdr->length) - sizeof (PTPIPHeader);
 	if (len < 0) {
-		gp_log (GP_LOG_ERROR, "ptpip/generic_read", "len < 0, %d?", len);
+		GP_LOG_E ("len < 0, %d?", len);
 		return PTP_RC_GeneralError;
 	}
 	*data = malloc (len);
 	if (!*data) {
-		gp_log (GP_LOG_ERROR, "ptpip/generic_read", "malloc failed.");
+		GP_LOG_E ("malloc failed.");
 		return PTP_RC_GeneralError;
 	}
 	curread = 0;
 	while (curread < len) {
 		ret = read (fd, (*data)+curread, len-curread);
 		if (ret == -1) {
-			gp_log (GP_LOG_ERROR, "ptpip/generic_read", "error %d in reading PTPIP data", errno);
+			GP_LOG_E ("error %d in reading PTPIP data", errno);
 			free (*data);*data = NULL;
 			return PTP_RC_GeneralError;
 		} else {
-			gp_log_data ( "ptpip/generic_read", (char*)((*data)+curread), ret);
+			GP_LOG_DATA ((char*)((*data)+curread), ret, "ptpip/generic_read data:");
 		}
 		if (ret == 0)
 			break;
 		curread += ret;
 	}
 	if (curread != len) {
-		gp_log (GP_LOG_ERROR, "ptpip/generic_read", "read PTPIP data, ret %d vs len %d", ret, len);
+		GP_LOG_E ("read PTPIP data, ret %d vs len %d", ret, len);
 		free (*data);*data = NULL;
 		return PTP_RC_GeneralError;
 	}
@@ -194,23 +207,16 @@ ptp_ptpip_evt_read (PTPParams* params, PTPIPHeader *hdr, unsigned char** data) {
 
 static uint16_t
 ptp_ptpip_check_event (PTPParams* params) {
-	fd_set		infds;
-	struct timeval	timeout;
-	int ret;
-	unsigned char*	data = NULL;
-	PTPIPHeader	hdr;
+	PTPContainer	event;
+	uint16_t	ret;
 
-	FD_ZERO(&infds);
-	FD_SET(params->evtfd, &infds);
-	timeout.tv_sec = 0;
-	timeout.tv_usec = 1;
-	if (1 != select (params->evtfd+1, &infds, NULL, NULL, &timeout))
-		return PTP_RC_OK;
-	ret = ptp_ptpip_evt_read (params, &hdr, &data);
+	event.Code = 0;
+	ret = ptp_ptpip_event (params, &event, PTP_EVENT_CHECK_FAST);
 	if (ret != PTP_RC_OK)
 		return ret;
-	gp_log (GP_LOG_DEBUG,"ptpip/check_event", "hdr type %d, length %d", hdr.type, hdr.length);
-	return PTP_RC_OK;
+	if (event.Code == 0)
+		return ret;
+	return ptp_add_event (params, &event);
 }
 
 #define ptpip_startdata_transid		0
@@ -225,20 +231,22 @@ ptp_ptpip_senddata (PTPParams* params, PTPContainer* ptp,
 		uint64_t size, PTPDataHandler *handler
 ) {
 	unsigned char	request[0x14];
-	int		ret, curwrite, towrite;
+	unsigned int	curwrite, towrite;
+	int		ret;
 	unsigned char*	xdata;
 
+	GP_LOG_D ("Sending PTP_OC 0x%0x (%s) data...", ptp->Code, ptp_get_opcode_name(params, ptp->Code));
 	htod32a(&request[ptpip_type],PTPIP_START_DATA_PACKET);
 	htod32a(&request[ptpip_len],sizeof(request));
 	htod32a(&request[ptpip_startdata_transid  + 8],ptp->Transaction_ID);
 	htod32a(&request[ptpip_startdata_totallen + 8],size);
 	htod32a(&request[ptpip_startdata_unknown  + 8],0);
-	gp_log_data ( "ptpip/senddata", (char*)request, sizeof(request));
+	GP_LOG_DATA ((char*)request, sizeof(request), "ptpip/senddata header:");
 	ret = write (params->cmdfd, request, sizeof(request));
 	if (ret == -1)
 		perror ("sendreq/write to cmdfd");
 	if (ret != sizeof(request)) {
-		gp_log (GP_LOG_ERROR, "ptpip/senddata", "ptp_ptpip_senddata() len=%d but ret=%d", (int)sizeof(request), ret);
+		GP_LOG_E ("ptp_ptpip_senddata() len=%d but ret=%d", (int)sizeof(request), ret);
 		return PTP_RC_GeneralError;
 	}
 	xdata = malloc(WRITE_BLOCKSIZE+8+4);
@@ -266,7 +274,7 @@ ptp_ptpip_senddata (PTPParams* params, PTPContainer* ptp,
 		htod32a(&xdata[ptpip_type], type);
 		htod32a(&xdata[ptpip_len], towrite2);
 		htod32a(&xdata[ptpip_data_transid+8], ptp->Transaction_ID);
-		gp_log_data("ptpip/senddata", (char*)xdata, towrite2);
+		GP_LOG_DATA ((char*)xdata, towrite2, "ptpip/senddata data:");
 		written = 0;
 		while (written < towrite2) {
 			ret = write (params->cmdfd, xdata+written, towrite2-written);
@@ -291,16 +299,17 @@ ptp_ptpip_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler
 	unsigned long		toread, curread;
 	int			xret;
 
+	GP_LOG_D ("Reading PTP_OC 0x%0x (%s) data...", ptp->Code, ptp_get_opcode_name(params, ptp->Code));
 	ret = ptp_ptpip_cmd_read (params, &hdr, &xdata);
 	if (ret != PTP_RC_OK)
 		return ret;
 
 	if (dtoh32(hdr.type) == PTPIP_CMD_RESPONSE) { /* might happen if we have no data transfer due to error? */
-		gp_log (GP_LOG_ERROR, "ptpip/getdata", "Unexpected ptp response, code %x", dtoh32a(&xdata[8]));
-		return PTP_RC_GeneralError;
+		GP_LOG_E ("Unexpected ptp response, ptp code %x", dtoh16a(&xdata[0]));
+		return dtoh16a(&xdata[0]);
 	}
 	if (dtoh32(hdr.type) != PTPIP_START_DATA_PACKET) {
-		gp_log (GP_LOG_ERROR, "ptpip/getdata", "got reply type %d\n", dtoh32(hdr.type));
+		GP_LOG_E ("got reply type %d\n", dtoh32(hdr.type));
 		return PTP_RC_GeneralError;
 	}
 	toread = dtoh32a(&xdata[ptpip_data_payload]);
@@ -311,50 +320,44 @@ ptp_ptpip_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler
 		if (ret != PTP_RC_OK)
 			return ret;
 		if (dtoh32(hdr.type) == PTPIP_END_DATA_PACKET) {
-			unsigned long written;
 			unsigned long datalen = dtoh32(hdr.length)-8-ptpip_data_payload;
 			if (datalen > (toread-curread)) {
-				gp_log (GP_LOG_ERROR, "ptpip/getdata",
-					"returned data is too much, expected %ld, got %ld",
-					(toread-curread),datalen
+				GP_LOG_E ("returned data is too much, expected %ld, got %ld",
+					  (toread-curread), datalen
 				);
 				break;
 			}
 			xret = handler->putfunc (params, handler->priv,
-				datalen, xdata+ptpip_data_payload, &written
+				datalen, xdata+ptpip_data_payload
 			);
-			if (xret == -1) {
-				gp_log (GP_LOG_ERROR, "ptpip/getdata",
-					"failed to putfunc of returned data");
+			if (xret != PTP_RC_OK) {
+				GP_LOG_E ("failed to putfunc of returned data");
 				break;
 			}
-			curread += written;
+			curread += datalen;
 			free (xdata); xdata = NULL;
 			continue;
 		}
 		if (dtoh32(hdr.type) == PTPIP_DATA_PACKET) {
-			unsigned long written;
 			unsigned long datalen = dtoh32(hdr.length)-8-ptpip_data_payload;
 			if (datalen > (toread-curread)) {
-				gp_log (GP_LOG_ERROR, "ptpip/getdata",
-					"returned data is too much, expected %ld, got %ld",
-					(toread-curread),datalen
+				GP_LOG_E ("returned data is too much, expected %ld, got %ld",
+					  (toread-curread), datalen
 				);
 				break;
 			}
 			xret = handler->putfunc (params, handler->priv,
-				datalen, xdata+ptpip_data_payload, &written
+				datalen, xdata+ptpip_data_payload
 			);
-			if (xret == -1) {
-				gp_log (GP_LOG_ERROR, "ptpip/getdata",
-					"failed to putfunc of returned data");
+			if (xret != PTP_RC_OK) {
+				GP_LOG_E ("failed to putfunc of returned data");
 				break;
 			}
-			curread += written;
+			curread += datalen;
 			free (xdata); xdata = NULL;
 			continue;
 		}
-		gp_log (GP_LOG_ERROR, "ptpip/getdata", "ret type %d", hdr.type);
+		GP_LOG_E ("ret type %d", hdr.type);
 	}
 	if (curread < toread)
 		return PTP_RC_GeneralError;
@@ -377,22 +380,38 @@ ptp_ptpip_getresp (PTPParams* params, PTPContainer* resp)
 	uint16_t 	ret;
 	int		n;
 
+	GP_LOG_D ("Reading PTP_OC 0x%0x (%s) response...", resp->Code, ptp_get_opcode_name(params, resp->Code));
+retry:
 	ret = ptp_ptpip_cmd_read (params, &hdr, &data);
 	if (ret != PTP_RC_OK)
 		return ret;
 
-	resp->Code		= dtoh16a(&data[ptpip_resp_code]);
-	resp->Transaction_ID	= dtoh32a(&data[ptpip_resp_transid]);
-	n = (dtoh32(hdr.length) - ptpip_resp_param1)/sizeof(uint32_t);
-	switch (n) {
-	case 5: resp->Param5 = dtoh32a(&data[ptpip_resp_param5]);
-	case 4: resp->Param4 = dtoh32a(&data[ptpip_resp_param4]);
-	case 3: resp->Param3 = dtoh32a(&data[ptpip_resp_param3]);
-	case 2: resp->Param2 = dtoh32a(&data[ptpip_resp_param2]);
-	case 1: resp->Param1 = dtoh32a(&data[ptpip_resp_param1]);
-	case 0: break;
+	switch (dtoh32(hdr.type)) {
+	case PTPIP_END_DATA_PACKET:
+		GP_LOG_D("PTPIP_END_DATA_PACKET");
+		resp->Transaction_ID	= dtoh32a(&data[0]);
+		free (data);
+		data = NULL;
+		goto retry;
+	case PTPIP_CMD_RESPONSE:
+		GP_LOG_D("PTPIP_CMD_RESPONSE");
+		resp->Code		= dtoh16a(&data[ptpip_resp_code]);
+		resp->Transaction_ID	= dtoh32a(&data[ptpip_resp_transid]);
+		n = (dtoh32(hdr.length) - sizeof(hdr) - ptpip_resp_param1)/sizeof(uint32_t);
+		switch (n) {
+		case 5: resp->Param5 = dtoh32a(&data[ptpip_resp_param5]);
+		case 4: resp->Param4 = dtoh32a(&data[ptpip_resp_param4]);
+		case 3: resp->Param3 = dtoh32a(&data[ptpip_resp_param3]);
+		case 2: resp->Param2 = dtoh32a(&data[ptpip_resp_param2]);
+		case 1: resp->Param1 = dtoh32a(&data[ptpip_resp_param1]);
+		case 0: break;
+		default:
+			GP_LOG_E ("response got %d parameters?", n);
+			break;
+		}
+		break;
 	default:
-		gp_log( GP_LOG_ERROR, "ptpip/getresp", "response got %d parameters?", n);
+		GP_LOG_E ("response type %d packet?", dtoh32(hdr.type));
 		break;
 	}
 	free (data);
@@ -407,7 +426,8 @@ ptp_ptpip_init_command_request (PTPParams* params)
 {
 	char		hostname[100];
 	unsigned char*	cmdrequest;
-	int 		len,i,ret;
+	unsigned int	i;
+	int 		len, ret;
 	unsigned char	guid[16];
 	
 	ptp_nikon_getptpipguid(guid);
@@ -428,16 +448,16 @@ ptp_ptpip_init_command_request (PTPParams* params)
 	htod16a(&cmdrequest[ptpip_initcmd_name+(strlen(hostname)+1)*2],PTPIP_VERSION_MINOR);
 	htod16a(&cmdrequest[ptpip_initcmd_name+(strlen(hostname)+1)*2+2],PTPIP_VERSION_MAJOR);
 
-	gp_log_data ( "ptpip/init_cmd", (char*)cmdrequest, len);
+	GP_LOG_DATA ((char*)cmdrequest, len, "ptpip/init_cmd data:");
 	ret = write (params->cmdfd, cmdrequest, len);
 	free (cmdrequest);
 	if (ret == -1) {
 		perror("write init cmd request");
 		return PTP_RC_GeneralError;
 	}
-	gp_log (GP_LOG_ERROR,"ptpip/init_cmd", "return %d / len %d", ret, len);
+	GP_LOG_E ("return %d / len %d", ret, len);
 	if (ret != len) {
-		gp_log (GP_LOG_ERROR, "ptpip", "return %d vs len %d", ret, len);
+		GP_LOG_E ("return %d vs len %d", ret, len);
 		return PTP_RC_GeneralError;
 	}
 	return PTP_RC_OK;
@@ -460,7 +480,10 @@ ptp_ptpip_init_command_ack (PTPParams* params)
 	if (ret != PTP_RC_OK)
 		return ret;
 	if (hdr.type != dtoh32(PTPIP_INIT_COMMAND_ACK)) {
-		gp_log (GP_LOG_ERROR, "ptpip/init_cmd_ack", "bad type returned %d", htod32(hdr.type));
+		GP_LOG_E ("bad type returned %d", htod32(hdr.type));
+		free (data);
+		if (hdr.type == PTPIP_INIT_FAIL) /* likely reason is permission denied */
+			return PTP_RC_AccessDenied;
 		return PTP_RC_GeneralError;
 	}
 	params->eventpipeid = dtoh32a(&data[ptpip_cmdack_idx]);
@@ -486,14 +509,14 @@ ptp_ptpip_init_event_request (PTPParams* params)
 	htod32a(&evtrequest[ptpip_len],ptpip_eventinit_size);
 	htod32a(&evtrequest[ptpip_eventinit_idx],params->eventpipeid);
 
-	gp_log_data ( "ptpip/init_event", (char*)evtrequest, ptpip_eventinit_size);
+	GP_LOG_DATA ((char*)evtrequest, ptpip_eventinit_size, "ptpip/init_event data:");
 	ret = write (params->evtfd, evtrequest, ptpip_eventinit_size);
 	if (ret == -1) {
 		perror("write init evt request");
 		return PTP_RC_GeneralError;
 	}
 	if (ret != ptpip_eventinit_size) {
-		gp_log (GP_LOG_ERROR, "ptpip", "unexpected retsize %d, expected %d", ret, ptpip_eventinit_size);
+		GP_LOG_E ("unexpected retsize %d, expected %d", ret, ptpip_eventinit_size);
 		return PTP_RC_GeneralError;
 	}
 	return PTP_RC_OK;
@@ -509,11 +532,11 @@ ptp_ptpip_init_event_ack (PTPParams* params)
 	ret = ptp_ptpip_evt_read (params, &hdr, &data);
 	if (ret != PTP_RC_OK)
 		return ret;
+	free (data);
 	if (hdr.type != dtoh32(PTPIP_INIT_EVENT_ACK)) {
-		gp_log (GP_LOG_ERROR, "ptpip", "bad type returned %d\n", htod32(hdr.type));
+		GP_LOG_E ("bad type returned %d\n", htod32(hdr.type));
 		return PTP_RC_GeneralError;
 	}
-	free (data);
 	return PTP_RC_OK;
 }
 
@@ -521,14 +544,75 @@ ptp_ptpip_init_event_ack (PTPParams* params)
 /* Event handling functions */
 
 /* PTP Events wait for or check mode */
-#define PTP_EVENT_CHECK			0x0000	/* waits for */
-#define PTP_EVENT_CHECK_FAST		0x0001	/* checks */
 
-static inline uint16_t
+#define ptpip_event_code    0
+#define ptpip_event_transid	2
+#define ptpip_event_param1	6
+#define ptpip_event_param2	10
+#define ptpip_event_param3	14
+
+static uint16_t
 ptp_ptpip_event (PTPParams* params, PTPContainer* event, int wait)
 {
-	fprintf(stderr,"event()\n");
+#ifndef WIN32
+	fd_set		infds;
+	struct timeval	timeout;
+	int ret;
+	unsigned char*	data = NULL;
+	PTPIPHeader	hdr;
+	int n;
+
+	while (1) {
+		FD_ZERO(&infds);
+		FD_SET(params->evtfd, &infds);
+		timeout.tv_sec = 0;
+		if (wait == PTP_EVENT_CHECK_FAST)
+			timeout.tv_usec = 1;
+		else
+			timeout.tv_usec = 1000; /* 1/1000 second  .. perhaps wait longer? */
+
+		ret = select (params->evtfd+1, &infds, NULL, NULL, &timeout);
+		if (1 != ret) {
+			if (-1 == ret) {
+				GP_LOG_D ("select returned error, errno is %d", errno);
+				return PTP_ERROR_IO;
+			}
+			return PTP_ERROR_TIMEOUT;
+		}
+
+		ret = ptp_ptpip_evt_read (params, &hdr, &data);
+		if (ret != PTP_RC_OK)
+			return ret;
+		GP_LOG_D ("hdr type %d, length %d", hdr.type, hdr.length);
+
+		if (dtoh32(hdr.type) == PTPIP_EVENT) {
+			break;
+		}
+
+		/* TODO: Handle cancel transaction and ping/pong
+		 * If not PTPIP_EVENT, process it and wait for next PTPIP_EVENT
+		 */
+		GP_LOG_E ("unknown/unhandled event type %d", dtoh32(hdr.type));
+	}
+
+	event->Code		= dtoh16a(&data[ptpip_event_code]);
+	event->Transaction_ID	= dtoh32a(&data[ptpip_event_transid]);
+	n = (dtoh32(hdr.length) - sizeof(hdr) - ptpip_event_param1)/sizeof(uint32_t);
+	switch (n) {
+	case 3: event->Param3 = dtoh32a(&data[ptpip_event_param3]);
+	case 2: event->Param2 = dtoh32a(&data[ptpip_event_param2]);
+	case 1: event->Param1 = dtoh32a(&data[ptpip_event_param1]);
+	case 0: break;
+	default:
+		GP_LOG_E ("response got %d parameters?", n);
+		break;
+	}
+	free (data);
 	return PTP_RC_OK;
+#else
+	GP_LOG_E ("not supported currently on Windows");
+	return PTP_RC_OK;
+#endif
 }
 
 uint16_t
@@ -605,19 +689,22 @@ ptp_nikon_getptpipguid (unsigned char* guid) {
 int
 ptp_ptpip_connect (PTPParams* params, const char *address) {
 	char 		*addr, *s, *p;
-	int		port;
+	int		port, tries;
 	struct sockaddr_in	saddr;
 	uint16_t	ret;
 
-	gp_log (GP_LOG_DEBUG,"ptpip/connect", "connecting to %s.", address);
+	GP_LOG_D ("connecting to %s.", address);
 	if (NULL == strchr (address,':'))
 		return GP_ERROR_BAD_PARAMETERS;
+
+#ifdef HAVE_INET_ATON
 	addr = strdup (address);
 	if (!addr)
 		return GP_ERROR_NO_MEMORY;
 	s = strchr (addr,':');
 	if (!s) {
-		gp_log (GP_LOG_ERROR,"ptpip/connect", "addr %s should contain a :", address);
+		GP_LOG_E ("addr %s should contain a :", address);
+		free (addr);
 		return GP_ERROR_BAD_PARAMETERS;
 	}
 	*s = '\0';
@@ -657,23 +744,42 @@ ptp_ptpip_connect (PTPParams* params, const char *address) {
 		return GP_ERROR_IO;
 	}
 	ret = ptp_ptpip_init_command_request (params);
-	if (ret != PTP_RC_OK)
+	if (ret != PTP_RC_OK) {
+		close (params->cmdfd);
+		close (params->evtfd);
 		return translate_ptp_result (ret);
+	}
 	ret = ptp_ptpip_init_command_ack (params);
-	if (ret != PTP_RC_OK)
+	if (ret != PTP_RC_OK) {
+		close (params->cmdfd);
+		close (params->evtfd);
 		return translate_ptp_result (ret);
-	if (-1 == connect (params->evtfd, (struct sockaddr*)&saddr, sizeof(struct sockaddr_in))) {
-		perror ("connect evt");
+	}
+	/* seen on Ricoh Theta, camera is not immediately ready. try again two times. */
+	tries = 2;
+	do {
+		if (-1 != connect (params->evtfd, (struct sockaddr*)&saddr, sizeof(struct sockaddr_in)))
+			break;
+		if ((errno == ECONNREFUSED) && (tries--)) {
+			GP_LOG_D ("event connect failed, retrying after short wait");
+			usleep(100*1000);
+			continue;
+		}
+		GP_LOG_E ("could not connect event");
 		close (params->cmdfd);
 		close (params->evtfd);
 		return GP_ERROR_IO;
-	}
+	} while (1);
 	ret = ptp_ptpip_init_event_request (params);
 	if (ret != PTP_RC_OK)
 		return translate_ptp_result (ret);
 	ret = ptp_ptpip_init_event_ack (params);
 	if (ret != PTP_RC_OK)
 		return translate_ptp_result (ret);
-	gp_log (GP_LOG_DEBUG, "ptpip/connect", "ptpip connected!");
+	GP_LOG_D ("ptpip connected!");
 	return GP_OK;
+#else
+	GP_LOG_E ("Windows currently not supported, neeeds a winsock port.");
+	return GP_ERROR_NOT_SUPPORTED;
+#endif
 }
