@@ -3,7 +3,7 @@
  * Implement Camera object representing a camera attached to the system.
  *
  * \author Copyright 2000 Scott Fritzinger
- * \author Copyright 2001-2002 Lutz Mller <lutz@users.sf.net>
+ * \author Copyright 2001-2002 Lutz Mueller <lutz@users.sf.net>
  *
  * \note
  * This library is free software; you can redistribute it and/or
@@ -20,8 +20,8 @@
  * \note
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -57,8 +57,6 @@
 #  define N_(String) (String)
 #endif
 
-#define CHECK_NULL(r)              {if (!(r)) return (GP_ERROR_BAD_PARAMETERS);}
-
 #define CAMERA_UNUSED(c,ctx)						\
 {									\
 	(c)->pc->used--;						\
@@ -75,17 +73,13 @@
 	int r1 = (result);						\
 									\
 	if (r1 < 0) {							\
-									\
 		/* libgphoto2_port doesn't have a GPContext */		\
-		if (r1 > -100)						\
-			gp_context_error ((ctx), _("An error occurred "	\
-				"in the io-library ('%s'): %s"),	\
-				gp_port_result_as_string (r1),		\
-				(c) ? gp_port_get_error ((c)->port) :	\
-				      _("No additional information "	\
-				      "available."));			\
+		gp_context_error ((ctx), _("An error occurred "		\
+			"in the io-library ('%s'): %s"),		\
+			gp_port_result_as_string (r1),			\
+			gp_port_get_error ((c) ? (c)->port : NULL));	\
 		if (c)							\
-			CAMERA_UNUSED((c),(ctx));			\
+			CAMERA_UNUSED (c,ctx);				\
 		return (r1);						\
 	}								\
 }
@@ -205,8 +199,8 @@
 	CHECK_OPEN (c,ctx);						\
 	r6 = (result);							\
 	if (r6 < 0) {							\
+		GP_LOG_E ("'%s' failed: %d", #result, r6);		\
 		CHECK_CLOSE (c,ctx);					\
-		gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Operation failed!");\
 		CAMERA_UNUSED (c,ctx);                              	\
 		return (r6);						\
 	}								\
@@ -269,10 +263,9 @@ struct _CameraPrivateCore {
 int
 gp_camera_exit (Camera *camera, GPContext *context)
 {
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Exiting camera ('%s')...",
-		camera->pc->a.model);
+	GP_LOG_D ("Exiting camera ('%s')...", camera->pc->a.model);
 
 	/*
 	 * We have to postpone this operation if the camera is currently 
@@ -300,8 +293,10 @@ gp_camera_exit (Camera *camera, GPContext *context)
 	memset (camera->functions, 0, sizeof (CameraFunctions));
 
 	if (camera->pc->lh) {
+#if !defined(VALGRIND)
 		lt_dlclose (camera->pc->lh);
 		lt_dlexit ();
+#endif
 		camera->pc->lh = NULL;
 	}
 
@@ -323,44 +318,34 @@ gp_camera_new (Camera **camera)
 {
 	int result;
 
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 
-        *camera = malloc (sizeof (Camera));
-	if (!*camera) 
-		return (GP_ERROR_NO_MEMORY);
-	memset (*camera, 0, sizeof (Camera));
+        C_MEM (*camera = calloc (1, sizeof (Camera)));
 
-        (*camera)->functions = malloc(sizeof(CameraFunctions));
-	if (!(*camera)->functions) {
-		gp_camera_free (*camera);
-		return (GP_ERROR_NO_MEMORY);
+        (*camera)->functions = calloc (1, sizeof (CameraFunctions));
+        (*camera)->pc        = calloc (1, sizeof (CameraPrivateCore));
+	if (!(*camera)->functions || !(*camera)->pc) {
+		result = GP_ERROR_NO_MEMORY;
+		goto error;
 	}
-	memset ((*camera)->functions, 0, sizeof (CameraFunctions));
-
-	(*camera)->pc = malloc (sizeof (CameraPrivateCore));
-	if (!(*camera)->pc) {
-		gp_camera_free (*camera);
-		return (GP_ERROR_NO_MEMORY);
-	}
-	memset ((*camera)->pc, 0, sizeof (CameraPrivateCore));
 
         (*camera)->pc->ref_count = 1;
 
 	/* Create the filesystem */
 	result = gp_filesystem_new (&(*camera)->fs);
-	if (result != GP_OK) {
-		gp_camera_free (*camera);
-		return (result);
-	}
+	if (result < GP_OK)
+		goto error;
 
 	/* Create the port */
 	result = gp_port_new (&(*camera)->port);
-	if (result < 0) {
-		gp_camera_free (*camera);
-		return (result);
-	}
+	if (result < GP_OK)
+		goto error;
 
         return(GP_OK);
+
+error:
+	gp_camera_free (*camera);
+	return result;
 }
 
 
@@ -382,10 +367,9 @@ gp_camera_new (Camera **camera)
 int
 gp_camera_set_abilities (Camera *camera, CameraAbilities abilities)
 {
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Setting abilities ('%s')...",
-		abilities.model);
+	GP_LOG_D ("Setting abilities ('%s')...", abilities.model);
 
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 
 	/*
 	 * If the camera is currently initialized, terminate that connection.
@@ -411,7 +395,7 @@ gp_camera_set_abilities (Camera *camera, CameraAbilities abilities)
 int
 gp_camera_get_abilities (Camera *camera, CameraAbilities *abilities)
 {
-	CHECK_NULL (camera && abilities);
+	C_PARAMS (camera && abilities);
 
 	memcpy (abilities, &camera->pc->a, sizeof (CameraAbilities));
 
@@ -422,7 +406,7 @@ gp_camera_get_abilities (Camera *camera, CameraAbilities *abilities)
 int
 gp_camera_get_port_info (Camera *camera, GPPortInfo *info)
 {
-	CHECK_NULL (camera && info);
+	C_PARAMS (camera && info);
 
 	CR (camera, gp_port_get_info (camera->port, info), NULL);
 
@@ -433,7 +417,8 @@ gp_camera_get_port_info (Camera *camera, GPPortInfo *info)
 int
 gp_camera_set_port_info (Camera *camera, GPPortInfo info)
 {
-	CHECK_NULL (camera);
+	char	*name, *path;
+	C_PARAMS (camera);
 
 	/*
 	 * If the camera is currently initialized, terminate that connection.
@@ -442,8 +427,9 @@ gp_camera_set_port_info (Camera *camera, GPPortInfo info)
 	if (camera->pc->lh)
 		gp_camera_exit (camera, NULL);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Setting port info for "
-		"port '%s' at '%s'...", info.name, info.path);
+	gp_port_info_get_name (info, &name);
+	gp_port_info_get_path (info, &path);
+	GP_LOG_D ("Setting port info for port '%s' at '%s'...", name, path);
 	CR (camera, gp_port_set_info (camera->port, info), NULL);
 
 	return (GP_OK);
@@ -471,19 +457,12 @@ gp_camera_set_port_speed (Camera *camera, int speed)
 {
 	GPPortSettings settings;
 
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 
-	if (!camera->port) {
-		gp_log (GP_LOG_ERROR, "camera", "You need to set "
-			"a port prior trying to set the speed");
-		return (GP_ERROR_BAD_PARAMETERS);
-	}
-
-	if (camera->port->type != GP_PORT_SERIAL) {
-		gp_log (GP_LOG_ERROR, "camera", "You can specify "
-			"a speed only with serial ports");
-		return (GP_ERROR_BAD_PARAMETERS);
-	}
+	C_PARAMS_MSG (camera->port,
+		      "You need to set a port prior trying to set the speed");
+	C_PARAMS_MSG (camera->port->type == GP_PORT_SERIAL,
+		      "You can specify a speed only with serial ports");
 
 	/*
 	 * If the camera is currently initialized, terminate that connection.
@@ -511,7 +490,7 @@ gp_camera_set_port_speed (Camera *camera, int speed)
 int
 gp_camera_get_port_speed (Camera *camera)
 {
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 
 	return (camera->pc->speed);
 }
@@ -527,7 +506,7 @@ gp_camera_get_port_speed (Camera *camera)
 int
 gp_camera_ref (Camera *camera)
 {
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 
 	camera->pc->ref_count += 1;
 
@@ -548,12 +527,11 @@ gp_camera_ref (Camera *camera)
 int
 gp_camera_unref (Camera *camera)
 {
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 
 	if (!camera->pc->ref_count) {
-		gp_log (GP_LOG_ERROR, "gphoto2-camera", "gp_camera_unref on "
-			"a camera with ref_count == 0 should not happen "
-			"at all");
+		GP_LOG_E ("gp_camera_unref on a camera with ref_count == 0 "
+			"should not happen at all");
 		return (GP_ERROR);
 	}
 
@@ -583,9 +561,9 @@ gp_camera_unref (Camera *camera)
 int
 gp_camera_free (Camera *camera)
 {
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 
-	gp_log (GP_LOG_DEBUG, "gp-camera", "Freeing camera...");
+	GP_LOG_D ("Freeing camera...");
 
 	/*
 	 * If the camera is currently initialized, close the connection.
@@ -601,8 +579,7 @@ gp_camera_free (Camera *camera)
 	}
 
 	if (camera->pc) {
-		if (camera->pc->timeout_ids)
-			free (camera->pc->timeout_ids);
+		free (camera->pc->timeout_ids);
 		free (camera->pc);
 		camera->pc = NULL;
 	}
@@ -622,6 +599,69 @@ gp_camera_free (Camera *camera)
 	return (GP_OK);
 }
 
+/**
+ * Autodetect all detectable camera
+ *
+ * @param list a #CameraList that receives the autodetected cameras
+ * @param context a #GPContext
+ * @return a gphoto2 error code
+ *
+ * This camera will autodetected all cameras that can be autodetected.
+ * This will for instance detect all USB cameras.
+ *
+ *   CameraList *list;
+ *   gp_list_new (&list);
+ *   gp_camera_autodetect (list, context);
+ *   ... done! ...
+ */
+int
+gp_camera_autodetect (CameraList *list, GPContext *context)
+{
+	CameraAbilitiesList	*al = NULL;
+	GPPortInfoList		*il = NULL;
+	int			ret, i;
+	CameraList		*xlist = NULL;
+
+	ret = gp_list_new (&xlist);
+	if (ret < GP_OK) goto out;
+	if (!il) {
+		/* Load all the port drivers we have... */
+		ret = gp_port_info_list_new (&il);
+		if (ret < GP_OK) goto out;
+		ret = gp_port_info_list_load (il);
+		if (ret < 0) goto out;
+		ret = gp_port_info_list_count (il);
+		if (ret < 0) goto out;
+	}
+	/* Load all the camera drivers we have... */
+	ret = gp_abilities_list_new (&al);
+	if (ret < GP_OK) goto out;
+	ret = gp_abilities_list_load (al, context);
+	if (ret < GP_OK) goto out;
+
+	/* ... and autodetect the currently attached cameras. */
+        ret = gp_abilities_list_detect (al, il, xlist, context);
+	if (ret < GP_OK) goto out;
+
+	/* Filter out the "usb:" entry */
+        ret = gp_list_count (xlist);
+	if (ret < GP_OK) goto out;
+	for (i=0;i<ret;i++) {
+		const char *name, *value;
+
+		gp_list_get_name (xlist, i, &name);
+		gp_list_get_value (xlist, i, &value);
+		if (!strcmp ("usb:",value)) continue;
+		gp_list_append (list, name, value);
+	}
+out:
+	if (il) gp_port_info_list_free (il);
+	if (al) gp_abilities_list_free (al);
+	gp_list_free (xlist);
+	if (ret < GP_OK)
+		return ret;
+	return gp_list_count(list);
+}
 
 /**
  * Initiate a connection to the \c camera. 
@@ -631,11 +671,11 @@ gp_camera_free (Camera *camera)
  * @return a gphoto2 error code
  *
  * Before calling this function, the
- * \c camera should be set up using #gp_camera_set_port_path or
- * #gp_camera_set_port_name and #gp_camera_set_abilities. If that has been
+ * \c camera should be set up using gp_camera_set_port_path() or
+ * gp_camera_set_port_name() and gp_camera_set_abilities(). If that has been
  * omitted, gphoto2 tries to autodetect any cameras and chooses the first one
  * if any cameras are found. It is generally a good idea to call
- * #gp_camera_exit after transactions have been completed in order to give
+ * gp_camera_exit() after transactions have been completed in order to give
  * other applications the chance to access the camera, too.
  *
  */
@@ -647,9 +687,9 @@ gp_camera_init (Camera *camera, GPContext *context)
 	CameraLibraryInitFunc init_func;
 	int result;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Initializing camera...");
+	GP_LOG_D ("Initializing camera...");
 
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 	/*
 	 * Reset the exit_requested flag. If this flag is set, 
 	 * gp_camera_exit will be called as soon as the camera is no
@@ -664,9 +704,10 @@ gp_camera_init (Camera *camera, GPContext *context)
 	if (strcasecmp (camera->pc->a.model, "Directory Browse") &&
 	    !strcmp ("", camera->pc->a.model)) {
 		CameraAbilitiesList *al;
-		GPPortInfo	pinfo;
 		GPPortInfoList	*il;
 		int		m, p;
+		char		*ppath;
+		GPPortType	ptype;
 		GPPortInfo	info;
         	CameraList	*list;
 
@@ -674,58 +715,56 @@ gp_camera_init (Camera *camera, GPContext *context)
 		if (result < GP_OK)
 			return result;
 
-		result = gp_port_get_info (camera->port, &pinfo);
-		if (result < GP_OK)
-			return result;
-
-		gp_log (GP_LOG_DEBUG, "gphoto2-camera", "pinfo.type %d", pinfo.type);
-		gp_log (GP_LOG_DEBUG, "gphoto2-camera", "pinfo.path %s", pinfo.path);
-		gp_log (GP_LOG_DEBUG, "gphoto2-camera", "pinfo.name %s", pinfo.name);
-		gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Neither "
-			"port nor model set. Trying auto-detection...");
+		GP_LOG_D ("Neither port nor model set. Trying auto-detection...");
 
 		/* Call auto-detect and choose the first camera */
-		gp_abilities_list_new (&al);
-		gp_abilities_list_load (al, context);
-		gp_port_info_list_new (&il);
-		gp_port_info_list_load (il);
-		gp_abilities_list_detect (al, il, list, context);
+		CRSL (camera, gp_abilities_list_new (&al), context, list);
+		CRSL (camera, gp_abilities_list_load (al, context), context, list);
+		CRSL (camera, gp_port_info_list_new (&il), context, list);
+		CRSL (camera, gp_port_info_list_load (il), context, list);
+		CRSL (camera, gp_abilities_list_detect (al, il, list, context), context, list);
 		if (!gp_list_count (list)) {
 			gp_abilities_list_free (al);
 			gp_port_info_list_free (il);
-			gp_context_error (context, _("Could not detect any camera"));
+			gp_context_error (context, _("Could not detect "
+					     "any camera"));
 			gp_list_free (list);
 			return (GP_ERROR_MODEL_NOT_FOUND);
 		}
 		p = 0;
-		/* if the port was set before, then use that entry. but not if it is "usb:" */
-		if (	(pinfo.type == GP_PORT_USB) &&
-			strlen(pinfo.path) &&
-			strcmp(pinfo.path,"usb:")
-		) {
+		CRSL (camera, gp_port_get_info (camera->port, &info), context, list);
+		CRSL (camera, gp_port_info_get_path (info, &ppath), context, list);
+		CRSL (camera, gp_port_info_get_type (info, &ptype), context, list);
+		/* if the port was set before, then use that entry, but not if it is "usb:" */
+		if ((ptype == GP_PORT_USB) && strlen(ppath) && strcmp(ppath, "usb:")) {
 			for (p = gp_list_count (list);p--;) {
 				const char *xp;
 
 				gp_list_get_value (list, p, &xp);
-				if (!strcmp (xp, pinfo.path))
+				if (!strcmp (xp, ppath))
 					break;
 			}
 			if (p<0) {
-				gp_context_error (context, _("Could not detect any camera at port %s"), pinfo.path);
+				gp_abilities_list_free (al);
+				gp_port_info_list_free (il);
+				gp_context_error (context, _("Could not detect any camera at port %s"), ppath);
+				gp_list_free (list);
 				return (GP_ERROR_FILE_NOT_FOUND);
 			}
 		}
 
-		gp_list_get_name  (list, p, &model);
+		CRSL (camera, gp_list_get_name  (list, p, &model), context, list);
 		m = gp_abilities_list_lookup_model (al, model);
-		gp_abilities_list_get_abilities (al, m, &a);
+		CRSL (camera, m, context, list);
+		CRSL (camera, gp_abilities_list_get_abilities (al, m, &a), context, list);
 		gp_abilities_list_free (al);
 		CRSL (camera, gp_camera_set_abilities (camera, a), context, list);
 		CRSL (camera, gp_list_get_value (list, p, &port), context, list);
 		p = gp_port_info_list_lookup_path (il, port);
-		gp_port_info_list_get_info (il, p, &info);
-		gp_port_info_list_free (il);
+		CRSL (camera, p, context, list);
+		CRSL (camera, gp_port_info_list_get_info (il, p, &info), context, list);
 		CRSL (camera, gp_camera_set_port_info (camera, info), context, list);
+		gp_port_info_list_free (il);
 		gp_list_free (list);
 	}
 
@@ -752,8 +791,7 @@ gp_camera_init (Camera *camera, GPContext *context)
 	}
 
 	/* Load the library. */
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Loading '%s'...",
-		camera->pc->a.library);
+	GP_LOG_D ("Loading '%s'...", camera->pc->a.library);
 	lt_dlinit ();
 	camera->pc->lh = lt_dlopenext (camera->pc->a.library);
 	if (!camera->pc->lh) {
@@ -819,12 +857,12 @@ gp_camera_init (Camera *camera, GPContext *context)
 int
 gp_camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 {
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 	CHECK_INIT (camera, context);
 
 	if (!camera->functions->get_config) {
 		gp_context_error (context, _("This camera does "
-			"not offer any configuration options."));
+			"not provide any configuration options."));
 		CAMERA_UNUSED (camera, context);
                 return (GP_ERROR_NOT_SUPPORTED);
 	}
@@ -837,6 +875,217 @@ gp_camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 }
 
 
+/**
+ * Retrieve a single configuration \c widget for the \c camera.
+ *
+ * @param camera a #Camera
+ * @param name the name of a configuration widget
+ * @param widget a #CameraWidget
+ * @param context a #GPContext
+ * @return gphoto2 error code
+ *
+ * This \c widget will then contain the current and the possible values and the type.
+ *
+ */
+int
+gp_camera_get_single_config (Camera *camera, const char *name, CameraWidget **widget, GPContext *context)
+{
+	CameraWidget		*rootwidget, *child;
+	CameraWidgetType	type;
+	const char		*label;
+	int			ret, ro;
+
+	C_PARAMS (camera);
+	CHECK_INIT (camera, context);
+
+	if (camera->functions->get_single_config) {
+		CHECK_RESULT_OPEN_CLOSE (camera, camera->functions->get_single_config (
+						camera, name, widget, context), context);
+
+		CAMERA_UNUSED (camera, context);
+		return GP_OK;
+	}
+
+	if (!camera->functions->get_config) {
+		gp_context_error (context, _("This camera does not provide any configuration options."));
+		CAMERA_UNUSED (camera, context);
+		return GP_ERROR_NOT_SUPPORTED;
+	}
+	/* emulate it ... */
+	CHECK_OPEN (camera, context);
+
+	ret = camera->functions->get_config ( camera, &rootwidget, context);
+	if (ret != GP_OK) {
+		CHECK_CLOSE (camera, context);
+		CAMERA_UNUSED (camera, context);
+		return ret;
+	}
+	ret = gp_widget_get_child_by_name (rootwidget, name, &child);
+	if (ret != GP_OK) {
+		gp_widget_free (rootwidget);
+		CHECK_CLOSE (camera, context);
+		CAMERA_UNUSED (camera, context);
+		return ret;
+	}
+
+	/* We need to duplicate the widget, as we will free the widgettree */
+	gp_widget_get_type (child, &type);
+	gp_widget_get_label (child, &label);
+	gp_widget_get_readonly (child, &ro);
+
+	ret = gp_widget_new (type, label, widget);
+	if (ret != GP_OK)
+		goto out;
+	gp_widget_set_name (*widget, name);
+	gp_widget_set_readonly (*widget, ro);
+
+	switch (type) {
+        case GP_WIDGET_MENU:
+        case GP_WIDGET_RADIO: {
+		char *value;
+		int i, nrofchoices;
+
+		nrofchoices = gp_widget_count_choices (child);
+		for (i = 0; i < nrofchoices; i++) {
+			const char *choice;
+
+			gp_widget_get_choice (child, i, &choice);
+			gp_widget_add_choice (*widget, choice);
+		}
+		gp_widget_get_value (child, &value);
+		gp_widget_set_value (*widget, value);
+		break;
+	}
+        case GP_WIDGET_TEXT: {
+		char *value;
+
+		gp_widget_get_value (child, &value);
+		gp_widget_set_value (*widget, value);
+		break;
+	}
+        case GP_WIDGET_RANGE: {
+		float value, rmin, rmax, rstep;
+
+		gp_widget_get_range (child, &rmin, &rmax, &rstep);
+		gp_widget_set_range (*widget, rmin, rmax, rstep);
+		gp_widget_get_value (child, &value);
+		gp_widget_set_value (*widget, &value);
+                break;
+	}
+        case GP_WIDGET_TOGGLE:
+        case GP_WIDGET_DATE: {
+		int value;
+
+		gp_widget_get_value (child, &value);
+		gp_widget_set_value (*widget, &value);
+                break;
+	}
+        case GP_WIDGET_BUTTON:
+        case GP_WIDGET_SECTION:
+        case GP_WIDGET_WINDOW:
+        default:
+                ret = GP_ERROR_BAD_PARAMETERS;
+		break;
+	}
+out:
+	gp_widget_free (rootwidget);
+	CHECK_CLOSE (camera, context);
+	CAMERA_UNUSED (camera, context);
+	return ret;
+}
+
+
+/**
+ * Retrieve a configuration \c list for the \c camera.
+ *
+ * @param camera a #Camera
+ * @param list a #CameraList
+ * @param context a #GPContext
+ * @return gphoto2 error code
+ *
+ * The names in list can be used for the single set and get configuration calls.
+ *
+ */
+
+static void
+_get_widget_names (CameraWidget *widget, CameraList *list)
+{
+	CameraWidgetType	type;
+
+	gp_widget_get_type (widget, &type);
+	switch (type) {
+        case GP_WIDGET_MENU:
+        case GP_WIDGET_RADIO:
+        case GP_WIDGET_TEXT:
+        case GP_WIDGET_RANGE:
+        case GP_WIDGET_TOGGLE:
+        case GP_WIDGET_DATE: {
+		const char *name;
+
+		gp_widget_get_name (widget, &name);
+		gp_list_append (list, name, NULL);
+                break;
+	}
+        case GP_WIDGET_SECTION:
+        case GP_WIDGET_WINDOW: {
+		int i, nrofchildren;
+
+		nrofchildren = gp_widget_count_children (widget);
+		for (i = 0; i < nrofchildren; i++) {
+			CameraWidget *child;
+
+			gp_widget_get_child (widget, i, &child);
+			_get_widget_names (child, list);
+		}
+		break;
+	}
+        case GP_WIDGET_BUTTON:
+        default:
+		break;
+	}
+
+}
+
+int
+gp_camera_list_config (Camera *camera, CameraList *list, GPContext *context)
+{
+	CameraWidget		*rootwidget;
+	int			ret;
+	C_PARAMS (camera);
+	CHECK_INIT (camera, context);
+
+	if (camera->functions->list_config) {
+		CHECK_RESULT_OPEN_CLOSE (camera, camera->functions->list_config (
+						camera, list, context), context);
+
+		CAMERA_UNUSED (camera, context);
+		return GP_OK;
+	}
+	if (!camera->functions->get_config) {
+		gp_context_error (context, _("This camera does not provide any configuration options."));
+		CAMERA_UNUSED (camera, context);
+		return GP_ERROR_NOT_SUPPORTED;
+	}
+	/* emulate it ... */
+	CHECK_OPEN (camera, context);
+
+	ret = camera->functions->get_config ( camera, &rootwidget, context);
+	if (ret != GP_OK) {
+		CHECK_CLOSE (camera, context);
+		CAMERA_UNUSED (camera, context);
+		return ret;
+	}
+
+	_get_widget_names (rootwidget, list);
+
+
+	gp_widget_free (rootwidget);
+	CHECK_CLOSE (camera, context);
+	CAMERA_UNUSED (camera, context);
+	return ret;
+}
+
+
 /**
  * Sets the configuration.
  *
@@ -852,7 +1101,7 @@ gp_camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
 int
 gp_camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 {
-	CHECK_NULL (camera && window);
+	C_PARAMS (camera && window);
 	CHECK_INIT (camera, context);
 
 	if (!camera->functions->set_config) {
@@ -869,6 +1118,103 @@ gp_camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 	return (GP_OK);
 }
 
+/**
+ * Set a single configuration \c widget for the \c camera.
+ *
+ * @param camera a #Camera
+ * @param name the name of a configuration widget
+ * @param widget a #CameraWidget
+ * @param context a #GPContext
+ * @return gphoto2 error code
+ *
+ * This \c widget contains the new value of the widget to set.
+ *
+ */
+int
+gp_camera_set_single_config (Camera *camera, const char *name, CameraWidget *widget, GPContext *context)
+{
+	CameraWidget		*rootwidget, *child;
+	CameraWidgetType	type;
+	int			ret;
+
+	C_PARAMS (camera);
+	CHECK_INIT (camera, context);
+
+	if (camera->functions->set_single_config) {
+		CHECK_RESULT_OPEN_CLOSE (camera, camera->functions->set_single_config (
+						camera, name, widget, context), context);
+
+		CAMERA_UNUSED (camera, context);
+		return GP_OK;
+	}
+
+	if (!camera->functions->set_config) {
+		gp_context_error (context, _("This camera does not provide any configuration options."));
+		CAMERA_UNUSED (camera, context);
+		return GP_ERROR_NOT_SUPPORTED;
+	}
+	/* emulate single config with the full tree */
+	CHECK_OPEN (camera, context);
+
+	ret = camera->functions->get_config ( camera, &rootwidget, context);
+	if (ret != GP_OK) {
+		CHECK_CLOSE (camera, context);
+		CAMERA_UNUSED (camera, context);
+		return ret;
+	}
+	ret = gp_widget_get_child_by_name (rootwidget, name, &child);
+	if (ret != GP_OK) {
+		gp_widget_free (rootwidget);
+		CHECK_CLOSE (camera, context);
+		CAMERA_UNUSED (camera, context);
+		return ret;
+	}
+
+	gp_widget_get_type (child, &type);
+	ret = GP_OK;
+	switch (type) {
+        case GP_WIDGET_MENU:
+        case GP_WIDGET_RADIO:
+        case GP_WIDGET_TEXT: {
+		char *value;
+
+		gp_widget_get_value (widget, &value);
+		gp_widget_set_value (child, value);
+		break;
+	}
+        case GP_WIDGET_RANGE: {
+		float value;
+
+		gp_widget_get_value (widget, &value);
+		gp_widget_set_value (child, &value);
+                break;
+	}
+        case GP_WIDGET_TOGGLE:
+        case GP_WIDGET_DATE: {
+		int value;
+
+		gp_widget_get_value (widget, &value);
+		gp_widget_set_value (child, &value);
+                break;
+	}
+        case GP_WIDGET_BUTTON:
+        case GP_WIDGET_SECTION:
+        case GP_WIDGET_WINDOW:
+        default:
+                ret = GP_ERROR_BAD_PARAMETERS;
+		break;
+	}
+	gp_widget_set_changed (child, 1);
+
+	if (ret == GP_OK)
+		ret = camera->functions->set_config (camera, rootwidget, context);
+	gp_widget_free (rootwidget);
+	CHECK_CLOSE (camera, context);
+	CAMERA_UNUSED (camera, context);
+	return ret;
+}
+
+
 /**
  * Retrieves a camera summary.
  *
@@ -884,7 +1230,7 @@ gp_camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 int
 gp_camera_get_summary (Camera *camera, CameraText *summary, GPContext *context)
 {
-	CHECK_NULL (camera && summary);
+	C_PARAMS (camera && summary);
 	CHECK_INIT (camera, context);
 
 	if (!camera->functions->summary) {
@@ -915,12 +1261,12 @@ gp_camera_get_summary (Camera *camera, CameraText *summary, GPContext *context)
 int
 gp_camera_get_manual (Camera *camera, CameraText *manual, GPContext *context)
 {
-	CHECK_NULL (camera && manual);
+	C_PARAMS (camera && manual);
 	CHECK_INIT (camera, context);
 
 	if (!camera->functions->manual) {
 		gp_context_error (context, _("This camera "
-			"does not offer a manual."));
+			"does not provide a manual."));
 		CAMERA_UNUSED (camera, context);
                 return (GP_ERROR_NOT_SUPPORTED);
 	}
@@ -947,7 +1293,7 @@ gp_camera_get_manual (Camera *camera, CameraText *manual, GPContext *context)
 int
 gp_camera_get_about (Camera *camera, CameraText *about, GPContext *context)
 {
-	CHECK_NULL (camera && about);
+	C_PARAMS (camera && about);
 	CHECK_INIT (camera, context);
 
 	if (!camera->functions->about) {
@@ -981,7 +1327,7 @@ int
 gp_camera_capture (Camera *camera, CameraCaptureType type,
 		   CameraFilePath *path, GPContext *context)
 {
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 	CHECK_INIT (camera, context);
 
 	if (!camera->functions->capture) {
@@ -997,6 +1343,34 @@ gp_camera_capture (Camera *camera, CameraCaptureType type,
 	return (GP_OK);
 }
 
+/**
+ * Triggers capture of one or more images.
+ *
+ * @param camera a #Camera
+ * @param context a #GPContext
+ * @return a gphoto2 error code
+ *
+ * This functions just remotely causes the shutter release and returns
+ * immediately. You will want to run #gp_camera_wait_event until a image
+ * is added which can be downloaded using #gp_camera_file_get.
+ **/
+int
+gp_camera_trigger_capture (Camera *camera, GPContext *context)
+{
+	C_PARAMS (camera);
+	CHECK_INIT (camera, context);
+
+	if (!camera->functions->trigger_capture) {
+		gp_context_error (context, _("This camera can not trigger capture."));
+		CAMERA_UNUSED (camera, context);
+                return (GP_ERROR_NOT_SUPPORTED);
+	}
+	CHECK_RESULT_OPEN_CLOSE (camera, camera->functions->trigger_capture (camera,
+						context), context);
+	CAMERA_UNUSED (camera, context);
+	return (GP_OK);
+}
+
 /**
  * Captures a preview that won't be stored on the camera but returned in 
  * supplied file. 
@@ -1006,14 +1380,15 @@ gp_camera_capture (Camera *camera, CameraCaptureType type,
  * @param context a #GPContext
  * @return a gphoto2 error code
  *
- * For example, you could use #gp_capture_preview for taking some sample
- * pictures before calling #gp_capture.
+ * For example, you could use gp_capture_preview() for taking some sample
+ * pictures before calling gp_capture().
  *
  **/
 int
 gp_camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 {
-	CHECK_NULL (camera && file);
+	char *xname;
+	C_PARAMS (camera && file);
 	CHECK_INIT (camera, context);
 
 	CR (camera, gp_file_clean (file), context);
@@ -1027,6 +1402,10 @@ gp_camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 
 	CHECK_RESULT_OPEN_CLOSE (camera, camera->functions->capture_preview (
 					camera, file, context), context);
+	gp_file_get_name_by_type (file, "capture_preview", GP_FILE_TYPE_NORMAL, &xname);
+	/* FIXME: Marcus ... will go away, just keep compatible now. */
+	gp_file_set_name (file, xname);
+	free (xname);
 
 	CAMERA_UNUSED (camera, context);
 	return (GP_OK);
@@ -1056,7 +1435,7 @@ gp_camera_wait_for_event (Camera *camera, int timeout,
 		          CameraEventType *eventtype, void **eventdata,
 			  GPContext *context)
 {
-	CHECK_NULL (camera);
+	C_PARAMS (camera);
 	CHECK_INIT (camera, context);
 
 	if (!camera->functions->wait_for_event) {
@@ -1084,10 +1463,9 @@ int
 gp_camera_folder_list_files (Camera *camera, const char *folder, 
 			     CameraList *list, GPContext *context)
 {
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Listing files in '%s'...",
-		folder);
+	GP_LOG_D ("Listing files in '%s'...", folder);
 
-	CHECK_NULL (camera && folder && list);
+	C_PARAMS (camera && folder && list);
 	CHECK_INIT (camera, context);
 	CR (camera, gp_list_reset (list), context);
 
@@ -1113,10 +1491,9 @@ int
 gp_camera_folder_list_folders (Camera *camera, const char* folder, 
 			       CameraList *list, GPContext *context)
 {
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Listing folders in '%s'...",
-		folder);
+	GP_LOG_D ("Listing folders in '%s'...", folder);
 
-	CHECK_NULL (camera && folder && list);
+	C_PARAMS (camera && folder && list);
 	CHECK_INIT (camera, context);
 	CR (camera, gp_list_reset (list), context);
 
@@ -1141,10 +1518,9 @@ int
 gp_camera_folder_delete_all (Camera *camera, const char *folder,
 			     GPContext *context)
 {
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Deleting all files in "
-		"'%s'...", folder);
+	GP_LOG_D ("Deleting all files in '%s'...", folder);
 
-	CHECK_NULL (camera && folder);
+	C_PARAMS (camera && folder);
 	CHECK_INIT (camera, context);
 
 	CHECK_RESULT_OPEN_CLOSE (camera, gp_filesystem_delete_all (camera->fs,
@@ -1165,17 +1541,19 @@ gp_camera_folder_delete_all (Camera *camera, const char *folder,
  *
  **/
 int
-gp_camera_folder_put_file (Camera *camera, const char *folder,
+gp_camera_folder_put_file (Camera *camera,
+			   const char *folder, const char *filename,
+			   CameraFileType type,
 			   CameraFile *file, GPContext *context)
 {
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Uploading file into '%s'...",
+	GP_LOG_D ("Uploading file into '%s'...",
 		folder);
 
-	CHECK_NULL (camera && folder && file);
+	C_PARAMS (camera && folder && file);
 	CHECK_INIT (camera, context);
 
 	CHECK_RESULT_OPEN_CLOSE (camera, gp_filesystem_put_file (camera->fs,
-					folder, file, context), context);
+					folder, filename, type, file, context), context);
 
 	CAMERA_UNUSED (camera, context);
 	return (GP_OK);
@@ -1203,10 +1581,9 @@ gp_camera_file_get_info (Camera *camera, const char *folder,
 	/* long int size; */
 	CameraFile *cfile;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Getting file info for '%s' "
-		"in '%s'...", file, folder);
+	GP_LOG_D ("Getting file info for '%s' in '%s'...", file, folder);
 
-	CHECK_NULL (camera && folder && file && info);
+	C_PARAMS (camera && folder && file && info);
 	CHECK_INIT (camera, context);
 
 	memset (info, 0, sizeof (CameraFileInfo));
@@ -1244,11 +1621,6 @@ gp_camera_file_get_info (Camera *camera, const char *folder,
 	}
 	gp_file_unref (cfile);
 
-	/* We don't trust the camera libraries */
-	info->file.fields |= GP_FILE_INFO_NAME;
-	strncpy (info->file.name, file, sizeof (info->file.name));
-	info->preview.fields &= ~GP_FILE_INFO_NAME;
-
 	CAMERA_UNUSED (camera, context);
 	return (GP_OK);
 }
@@ -1269,7 +1641,7 @@ gp_camera_file_set_info (Camera *camera, const char *folder,
 			 const char *file, CameraFileInfo info,
 			 GPContext *context)
 {
-	CHECK_NULL (camera && folder && file);
+	C_PARAMS (camera && folder && file);
 	CHECK_INIT (camera, context);
 
 	CHECK_RESULT_OPEN_CLOSE (camera, gp_filesystem_set_info (camera->fs,
@@ -1296,10 +1668,9 @@ gp_camera_file_get (Camera *camera, const char *folder, const char *file,
 		    CameraFileType type, CameraFile *camera_file,
 		    GPContext *context)
 {
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Getting file '%s' in "
-		"folder '%s'...", file, folder);
+	GP_LOG_D ("Getting file '%s' in folder '%s'...", file, folder);
 
-	CHECK_NULL (camera && folder && file && camera_file);
+	C_PARAMS (camera && folder && file && camera_file);
 	CHECK_INIT (camera, context);
 
 	CR (camera, gp_file_clean (camera_file), context);
@@ -1321,6 +1692,48 @@ gp_camera_file_get (Camera *camera, const char *folder, const char *file,
 	return (GP_OK);
 }
 
+/**
+ * Reads a file partially from the #Camera.
+ *
+ * @param camera a #Camera
+ * @param folder a folder
+ * @param file the name of a file
+ * @param type the #CameraFileType
+ * @param offset the offset into the camera file
+ * @param data the buffer receiving the data
+ * @param size the size to be read and that was read
+ * @param context a #GPContext
+ * @return a gphoto2 error code
+ *
+ **/
+int
+gp_camera_file_read (Camera *camera, const char *folder, const char *file,
+		    CameraFileType type,
+		    uint64_t offset, char *buf, uint64_t *size,
+		    GPContext *context)
+{
+	GP_LOG_D ("Getting file '%s' in folder '%s'...", file, folder);
+
+	C_PARAMS (camera && folder && file && buf && size);
+	CHECK_INIT (camera, context);
+
+	/* Did we get reasonable foldername/filename? */
+	if (strlen (folder) == 0) {
+		CAMERA_UNUSED (camera, context);
+		return (GP_ERROR_DIRECTORY_NOT_FOUND);
+	}
+	if (strlen (file) == 0) {
+		CAMERA_UNUSED (camera, context);
+		return (GP_ERROR_FILE_NOT_FOUND);
+	}
+  
+	CHECK_RESULT_OPEN_CLOSE (camera, gp_filesystem_read_file (camera->fs,
+			folder, file, type, offset, buf, size, context), context);
+
+	CAMERA_UNUSED (camera, context);
+	return (GP_OK);
+}
+
 /**
  * Deletes the file from \c folder.
  *
@@ -1335,10 +1748,9 @@ int
 gp_camera_file_delete (Camera *camera, const char *folder, const char *file,
 		       GPContext *context)
 {
-	gp_log (GP_LOG_DEBUG, "gphoto2-camera", "Deleting file '%s' in "
-		"folder '%s'...", file, folder);
+	GP_LOG_D ("Deleting file '%s' in folder '%s'...", file, folder);
 
-	CHECK_NULL (camera && folder && file);
+	C_PARAMS (camera && folder && file);
 	CHECK_INIT (camera, context);
 
 	CHECK_RESULT_OPEN_CLOSE (camera, gp_filesystem_delete_file (
@@ -1362,7 +1774,7 @@ int
 gp_camera_folder_make_dir (Camera *camera, const char *folder,
 			   const char *name, GPContext *context)
 {
-	CHECK_NULL (camera && folder && name);
+	C_PARAMS (camera && folder && name);
 	CHECK_INIT (camera, context);
 
 	CHECK_RESULT_OPEN_CLOSE (camera, gp_filesystem_make_dir (camera->fs,
@@ -1386,7 +1798,7 @@ int
 gp_camera_folder_remove_dir (Camera *camera, const char *folder,
 			     const char *name, GPContext *context)
 {
-	CHECK_NULL (camera && folder && name);
+	C_PARAMS (camera && folder && name);
 	CHECK_INIT (camera, context);
 
 	CHECK_RESULT_OPEN_CLOSE (camera, gp_filesystem_remove_dir (camera->fs,
@@ -1422,7 +1834,7 @@ gp_camera_get_storageinfo (
 	Camera *camera, CameraStorageInformation **sifs,
 	int *nrofsifs, GPContext *context)
 {
-	CHECK_NULL (camera && sifs && nrofsifs);
+	C_PARAMS (camera && sifs && nrofsifs);
 	CHECK_INIT (camera, context);
 
 	CHECK_RESULT_OPEN_CLOSE (camera,
@@ -1478,10 +1890,8 @@ gp_camera_start_timeout (Camera *camera, unsigned int timeout,
 			 CameraTimeoutFunc func)
 {
 	int id;
-	unsigned int *ids;
 
-	if (!camera || !camera->pc)
-		return (GP_ERROR_BAD_PARAMETERS);
+	C_PARAMS (camera && camera->pc);
 
 	if (!camera->pc->timeout_start_func)
 		return (GP_ERROR_NOT_SUPPORTED);
@@ -1490,11 +1900,9 @@ gp_camera_start_timeout (Camera *camera, unsigned int timeout,
 	 * We remember the id here in order to automatically remove
 	 * the timeout on gp_camera_exit.
 	 */
-	ids = realloc (camera->pc->timeout_ids, sizeof (int) *
-					(camera->pc->timeout_ids_len + 1));
-	if (!ids)
-		return (GP_ERROR_NO_MEMORY);
-	camera->pc->timeout_ids = ids;
+	C_MEM (camera->pc->timeout_ids =
+			realloc (camera->pc->timeout_ids, sizeof (int) *
+					(camera->pc->timeout_ids_len + 1)));
 
 	id = camera->pc->timeout_start_func (camera, timeout, func,
 					     camera->pc->timeout_data);
