@@ -17,8 +17,8 @@
  * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #define _BSD_SOURCE
@@ -29,14 +29,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 
 #include <gphoto2/gphoto2-result.h>
 #include <gphoto2/gphoto2-port-log.h>
 #include <gphoto2/gphoto2-port-portability.h>
 
-#define GP_MODULE "setting"
-
 /**
  * Internal struct to store settings.
  */
@@ -53,7 +50,6 @@ static Setting         glob_setting[512];
 
 static int save_settings (void);
 
-#define CHECK_NULL(r)              {if (!(r)) return (GP_ERROR_BAD_PARAMETERS);}
 #define CHECK_RESULT(result)       {int r = (result); if (r < 0) return (r);}
 
 static int load_settings (void);
@@ -73,7 +69,7 @@ gp_setting_get (char *id, char *key, char *value)
 {
         int x;
 
-	CHECK_NULL (id && key);
+	C_PARAMS (id && key);
 
 	if (!glob_setting_count)
 		load_settings ();
@@ -105,13 +101,12 @@ gp_setting_set (char *id, char *key, char *value)
 {
         int x;
 
-	CHECK_NULL (id && key);
+	C_PARAMS (id && key);
 
 	if (!glob_setting_count)
 		load_settings ();
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-setting",
-		"Setting key '%s' to value '%s' (%s)", key, value, id);
+	GP_LOG_D ("Setting key '%s' to value '%s' (%s)", key, value, id);
 
         for (x=0; x<glob_setting_count; x++) {
                 if ((strcmp(glob_setting[x].id, id)==0) &&
@@ -134,10 +129,10 @@ verify_settings (char *settings_file)
 {
 	FILE *f;
 	char buf[1024];
-	int x, equals;
+	unsigned int x, equals;
 
 	if ((f=fopen(settings_file, "r"))==NULL) {
-		GP_DEBUG ("Can't open settings file for reading");
+		GP_LOG_D ("Can't open settings file '%s' for reading.", settings_file);
 		return(0);
 	}
 
@@ -155,7 +150,7 @@ verify_settings (char *settings_file)
 
 			if (equals < 2) {
 				fclose (f);
-				GP_DEBUG ("Incorrect settings format. resetting\n");
+				GP_LOG_E ("Incorrect settings format. Resetting.");
 				unlink(settings_file);
 				return (GP_ERROR);
 			}
@@ -173,13 +168,13 @@ load_settings (void)
 	char buf[1024], *id, *key, *value;
 
 	/* Make sure the directories are created */
-	GP_DEBUG ("Creating $HOME/.gphoto");
 #ifdef WIN32
 	GetWindowsDirectory (buf, sizeof(buf));
 	strcat (buf, "\\gphoto");
 #else
 	snprintf (buf, sizeof(buf), "%s/.gphoto", getenv ("HOME"));
 #endif
+	GP_LOG_D ("Creating gphoto config directory ('%s')", buf);
 	(void)gp_system_mkdir (buf);
 
 	glob_setting_count = 0;
@@ -193,10 +188,10 @@ load_settings (void)
 	if (verify_settings(buf) != GP_OK)
 		/* verify_settings will unlink and recreate the settings file */
 		return (GP_OK);
-	GP_DEBUG ("Loading settings from file \"%s\"", buf);
+	GP_LOG_D ("Loading settings from file '%s'.", buf);
 
 	if ((f=fopen(buf, "r"))==NULL) {
-		GP_DEBUG ("Can't open settings for reading");
+		GP_LOG_D ("Can't open settings file '%s' for reading.", buf);
 		return(GP_ERROR);
 	}
 
@@ -232,12 +227,10 @@ save_settings (void)
 
 	snprintf (buf, sizeof(buf), "%s/.gphoto/settings", getenv ("HOME"));
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-setting",
-		"Saving %i setting(s) to file \"%s\"",
-		glob_setting_count, buf);
+	GP_LOG_D ("Saving %i setting(s) to file \"%s\"", glob_setting_count, buf);
 
 	if ((f=fopen(buf, "w+"))==NULL) {
-		GP_DEBUG ("Can't open settings file for writing");
+		GP_LOG_E ("Can't open settings file for writing.");
 		return(0);
 	}
 	rewind(f);
